#define S_FUNCTION_NAME YUVmask
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"
#include "math.h"
#include "../../Global_par/constants.h"

static void mdlInitializeSizes(SimStruct *S)
{
    DECL_AND_INIT_DIMSINFO(inputDimsInfo);
    DECL_AND_INIT_DIMSINFO(outputDimsInfo);
    
    /*PARAMETERS*/
    ssSetNumSFcnParams(S, 0);
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }
    
    /*INPUT PORTS*/
    if (!ssSetNumInputPorts(S, 4)) return;
    
    inputDimsInfo.width = SIZEX;
    ssSetInputPortDimensionInfo(S, 0, &inputDimsInfo);
    ssSetInputPortMatrixDimensions(  S ,0, SIZEX, SIZEY);
    ssSetInputPortDataType(S, 0, SS_UINT8);
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortRequiredContiguous(S, 0, 1);
    
    inputDimsInfo.width = SIZEX;
    ssSetInputPortDimensionInfo(S, 1, &inputDimsInfo);
    ssSetInputPortMatrixDimensions(  S ,1, SIZEX, SIZEY);
    ssSetInputPortDataType(S, 1, SS_UINT8);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
    ssSetInputPortRequiredContiguous(S, 1, 1);
    
    inputDimsInfo.width = SIZEX;
    ssSetInputPortDimensionInfo(S, 2, &inputDimsInfo);
    ssSetInputPortMatrixDimensions(  S ,2, SIZEX, SIZEY);
    ssSetInputPortDataType(S, 2, SS_UINT8);
    ssSetInputPortDirectFeedThrough(S, 2, 1);
    ssSetInputPortRequiredContiguous(S, 2, 1);
    
    inputDimsInfo.width = SIZEX;
    ssSetInputPortDimensionInfo(S, 3, &inputDimsInfo);
    ssSetInputPortMatrixDimensions(  S ,3, SIZEX, SIZEY);
    ssSetInputPortDataType(S, 3, SS_UINT8);
    ssSetInputPortDirectFeedThrough(S, 3, 1);
    ssSetInputPortRequiredContiguous(S, 3, 1);
    
    /*OUTPUT PORTS*/
    if (!ssSetNumOutputPorts(S, 3)) return;
    
    outputDimsInfo.width = SIZEX;
    ssSetOutputPortDimensionInfo(S, 0, &outputDimsInfo);
    ssSetOutputPortMatrixDimensions( S ,0, SIZEX, SIZEY);
    ssSetOutputPortDataType(S, 0, SS_UINT8);
    
    outputDimsInfo.width = SIZEX;
    ssSetOutputPortDimensionInfo(S, 1, &outputDimsInfo);
    ssSetOutputPortMatrixDimensions( S ,1, SIZEX, SIZEY);
    ssSetOutputPortDataType(S, 1, SS_UINT8);
    
    outputDimsInfo.width = SIZEX;
    ssSetOutputPortDimensionInfo(S, 2, &outputDimsInfo);
    ssSetOutputPortMatrixDimensions( S ,2, SIZEX, SIZEY);
    ssSetOutputPortDataType(S, 2, SS_UINT8);
}

static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}

static void mdlOutputs(SimStruct *S, int_T tid)
{
    /*inputs*/
    /*goalx contains [distance angle] of goal x*/
    const uint8_T   *Yin  = (const uint8_T*) ssGetInputPortSignal(S,0);
    const uint8_T   *Uin  = (const uint8_T*) ssGetInputPortSignal(S,1);
    const uint8_T   *Vin  = (const uint8_T*) ssGetInputPortSignal(S,2);
    const uint8_T   *mask  = (const uint8_T*) ssGetInputPortSignal(S,3);
    
    /*outputs*/
    uint8_T *Yout  = (uint8_T *)ssGetOutputPortRealSignal(S,0);
    uint8_T *Uout  = (uint8_T *)ssGetOutputPortRealSignal(S,1);
    uint8_T *Vout  = (uint8_T *)ssGetOutputPortRealSignal(S,2);
    
    int i;
    for (i=0;i<NPIXELS;++i)
    {
        Yout[i] = Yin[i] & mask[i];
        Uout[i] = Uin[i] & mask[i];
        Vout[i] = Vin[i] & mask[i];
    }
}

#define MDL_SET_INPUT_PORT_DIMENSION_INFO
static void mdlSetInputPortDimensionInfo(SimStruct        *S,
int_T            port,
const DimsInfo_T *dimsInfo)
{
    if(!ssSetInputPortDimensionInfo(S, port, dimsInfo)) return;
}

# define MDL_SET_OUTPUT_PORT_DIMENSION_INFO
static void mdlSetOutputPortDimensionInfo(SimStruct        *S,
int_T            port,
const DimsInfo_T *dimsInfo)
{
    if(!ssSetOutputPortDimensionInfo(S, port, dimsInfo)) return;
}

static void mdlTerminate(SimStruct *S)
{
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif


