#define S_FUNCTION_NAME adaptivepolygonmask
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"
#include "math.h"
#include "../../Global_par/constants.h"

#define REMOVEX (3+.5*FIELDWIDTH)
#define REMOVEY (2+.5*FIELDLENGTH)
static double centerx,centery;
static double par_a,par_b;
static real_T correctionx,correctiony,correctionphi;
static double mc[9];
/********************************************************/

static double wrappen(double in)
{
    in = fmod(in,2.0*M_PI);
    if (in<0) in+=2.0*M_PI;
    if (in>M_PI) in-=2.0*M_PI;
    return in;
}

int toppixel(int *px)
{
    int i,pxmin=px[0];
    int x0ind = 0;
    for (i=1;i<NANGLES;++i)
    {
        if (px[i]<pxmin)
        {
            x0ind = i;
            pxmin = px[i];
        }
    }
    return x0ind;
}

int bottompixel(int *px)
{
    int i,pxmax=px[0];
    int x1ind = 0;
    for (i=1;i<NANGLES;++i)
    {
        if (px[i]>pxmax)
        {
            x1ind = i;
            pxmax = px[i];
        }
    }
    return x1ind;
}

void leftside(int *yleft,int *px,int *py,int x0ind, int x1ind)
{
    int i,j,k,nint;
    for (i=0;i<SIZEX;++i) yleft[i]=SIZEY-1;
    
    j=x0ind;
    while (j!=x1ind)
    {
        k = j+1;
        if (k==NANGLES) k=0;
        nint = px[k]-px[j];
        if (nint==0)
        {
            if (yleft[px[j]]>py[j])
            {
                yleft[px[j]] = py[j];
            }
        }
        else
        {
            for (i=0;i<nint;++i)
            {
                yleft[px[j]+i] = (py[j]*(nint-i)+py[k]*i)/nint;
            }
        }
        ++j;
        if (j==NANGLES) j=0;
    }
}

void rightside(int *yright,int *px,int *py,int x0ind, int x1ind)
{
    int i,j,nint,k;
    for (i=0;i<SIZEX;++i) yright[i]=0;
    
    j=x0ind;
    while (j!=x1ind)
    {
        k=j-1;
        if (k==-1) k=NANGLES-1;
        nint = px[k]-px[j];
        if (nint==0)
        {
            if (yright[px[j]]<py[j])
            {
                yright[px[j]] = py[j];
            }
        }
        else
        {
            for (i=0;i<nint;++i)
            {
                yright[px[j]+i] = (py[j]*(nint-i)+py[k]*i)/nint;
            }
        }
        --j;
        if (j==-1) j=NANGLES-1;
    }
}

void filllines(uint8_T *mask, int *yleft, int *yright)
{
    int i,j;
    for (j=0;j<SIZEY;++j) for (i=0;i<SIZEX;++i) if (!((j>yleft[i]) && (j<yright[i]))) mask[i+j*SIZEX]=0;
}

void fillpolygon(uint8_T *mask, int *edge)
{
    int x0ind,x1ind;
    int yright[SIZEX],yleft[SIZEX];
    int *px,*py;
    
    px = edge;
    py = edge+NANGLES;
    
    x0ind = toppixel(px);
    x1ind = bottompixel(px);
    leftside(yleft,px,py,x0ind,x1ind);
    rightside(yright,px,py,x0ind,x1ind); /*1.3 ms*/
    filllines(mask,yleft,yright);
}

void getedge(int_T* edge,int_T* edgeradius,const real_T* posevision)
{
    int i,i_image,i_delta;
    double fac = M_PI*2/NANGLES,a,tr,pose[3];
    /*get pose vision*/
    pose[0] = posevision[0];
    pose[1] = posevision[1];
    pose[2] = posevision[2];
    
    /*transform pose position from robot center to linepoints center*/
    pose[0]-=correctionx*cos(pose[2])-correctiony*sin(pose[2]);
    pose[1]-=correctionx*sin(pose[2])+correctiony*cos(pose[2]);
    pose[2]-=correctionphi;
    pose[2]=wrappen(pose[2]);
    
    /*clip posee to edge of polygon mask*/
    if (pose[0]>REMOVEX*.999) pose[0]=REMOVEX*.999;
    if (pose[0]<-REMOVEX*.999) pose[0]=-REMOVEX*.999;
    if (pose[1]>REMOVEY*.999) pose[1]=REMOVEY*.999;
    if (pose[1]<-REMOVEY*.999) pose[1]=-REMOVEY*.999;
    
    /*get edge points in NANGLES uniform directions*/
    i_delta = (int)(pose[2]/fac);
    if (i_delta<0) i_delta+=NANGLES;
    for (i=0;i<NANGLES;++i)
    {
        /*mapping from global position to image position*/
        a = wrappen(i*fac);
        if (a>=0 && a<.5*M_PI)
            if (a>atan2(pose[0]+REMOVEX,REMOVEY-pose[1])) tr=(pose[0]+REMOVEX)/sin(a);
            else tr=(REMOVEY-pose[1])/cos(a);
        else if (a>=.5*M_PI)
            if (a<atan2(pose[0]+REMOVEX,-REMOVEY-pose[1])) tr=(pose[0]+REMOVEX)/sin(a);
            else tr=(-REMOVEY-pose[1])/cos(a);
        else if (a<-.5*M_PI)
            if (a>atan2(pose[0]-REMOVEX,-REMOVEY-pose[1])) tr=(pose[0]-REMOVEX)/sin(a);
            else tr=-(REMOVEY+pose[1])/cos(a);
        else
            if (a<atan2(pose[0]-REMOVEX,REMOVEY-pose[1])) tr=(pose[0]-REMOVEX)/sin(a);
            else tr=-(-REMOVEY+pose[1])/cos(a);
        
        /*save edge radius, pose correction*/
        i_image = i-i_delta;
        if (i_image>=NANGLES) i_image-=NANGLES;
        if (i_image<0) i_image+=NANGLES;
        edgeradius[i_image] = atan(tr/par_a)/par_b;
        
        /*save edge xy*/
        edge[i] = (int)(centerx-edgeradius[i_image]*cos(a-pose[2]));
        edge[i+NANGLES] = (int)(centery-edgeradius[i_image]*sin(a-pose[2]));
    }
}

/********************************************************/


static void mdlInitializeSizes(SimStruct *S)
{
    DECL_AND_INIT_DIMSINFO(inputDimsInfo);
    DECL_AND_INIT_DIMSINFO(outputDimsInfo);
    
    FILE *fid;
    
    /*PARAMETERS*/
    ssSetNumSFcnParams(S, 0);
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }
    
    fid=fopen("mc.dat","rb");
    if (fid!=NULL)
    {
        fread(mc,sizeof(double),9,fid);
        fclose(fid);
        
        centerx = mc[0];
        centery = mc[1];
        par_a = mc[2];
        par_b = mc[3];
        correctionx = mc[4];
        correctiony = mc[5];
        correctionphi = mc[8];
    }
    else
    {
        printf("could not find mc.dat!!!\n");
        return;
    }
    
    /*INPUT PORTS*/
    if (!ssSetNumInputPorts(S, 3)) return;
    
    inputDimsInfo.width = SIZEX;
    ssSetInputPortDimensionInfo(S, 0, &inputDimsInfo);
    ssSetInputPortMatrixDimensions(  S ,0, SIZEX, SIZEY);
    ssSetInputPortDataType(S, 0, SS_UINT8);
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortRequiredContiguous(S, 0, 1);
    
    ssSetInputPortWidth(S, 1, 3);
    ssSetInputPortDataType(S, 1, SS_DOUBLE);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
    ssSetInputPortRequiredContiguous(S, 1, 1);
    
    ssSetInputPortWidth(S, 2, 1);
    ssSetInputPortDataType(S, 2, SS_INT32);
    ssSetInputPortDirectFeedThrough(S, 2, 1);
    ssSetInputPortRequiredContiguous(S, 2, 1);
    
    /*OUTPUT PORTS*/
    if (!ssSetNumOutputPorts(S, 2)) return;
    
    outputDimsInfo.width = SIZEX;
    ssSetOutputPortDimensionInfo(S, 0, &outputDimsInfo);
    ssSetOutputPortMatrixDimensions( S ,0, SIZEX, SIZEY);
    ssSetOutputPortDataType(S, 0, SS_UINT8);
    
    ssSetOutputPortWidth(S, 1, NANGLES);
    ssSetOutputPortDataType(S, 1, SS_INT32);
}

static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}

static void mdlOutputs(SimStruct *S, int_T tid)
{
    /*inputs*/
    /*goalx contains [distance angle] of goal x*/
    const uint8_T   *maskinput  = (const uint8_T*) ssGetInputPortSignal(S,0);
    const real_T *posevision  = (const real_T*) ssGetInputPortSignal(S,1);
    const int_T *posefound  = (const int_T*) ssGetInputPortSignal(S,2);
    
    /*outputs*/
    uint8_T *maskoutput  = (uint8_T *)ssGetOutputPortRealSignal(S,0);
    int_T *edgeradius  = (int_T *)ssGetOutputPortRealSignal(S,1);
    
    int i,edge[2*NANGLES];
    for (i=0;i<SIZEX*SIZEY;++i) maskoutput[i]=maskinput[i]; 
    
    if (posefound[0]==1)
    {
        getedge(edge,edgeradius,posevision);  
        fillpolygon(maskoutput,edge);
    }
    else
    {
        for (i=0;i<NANGLES;++i) edgeradius[i]=200;
    }
}

#define MDL_SET_INPUT_PORT_DIMENSION_INFO
static void mdlSetInputPortDimensionInfo(SimStruct        *S,
int_T            port,
const DimsInfo_T *dimsInfo)
{
    if(!ssSetInputPortDimensionInfo(S, port, dimsInfo)) return;
}

# define MDL_SET_OUTPUT_PORT_DIMENSION_INFO
static void mdlSetOutputPortDimensionInfo(SimStruct        *S,
int_T            port,
const DimsInfo_T *dimsInfo)
{
    if(!ssSetOutputPortDimensionInfo(S, port, dimsInfo)) return;
}

static void mdlTerminate(SimStruct *S)
{
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif


