#define S_FUNCTION_NAME ballsizefilter
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"
#include "math.h"
#include "../../Global_par/constants.h"

#define MAXDISTANCEPIX 200 /*pixels maximum range to detect balls*/
#define MINDISTANCEPIX 30 /*pixels maximum range to detect balls*/
#define MINKERNEL 3
#define MAXKERNEL 9
#define MAXCONVSIZE ((MAXKERNEL*2+1)*(MAXKERNEL*2+1)) 
#define NSTEP 4
#define KERNELGRID 1
#define KERNELNOISE 0
#define KERNELFAC 0.5
#define LABELINDEX 1
#define FUNC(i) {for (j=0,sel = convdataind[i],siz = convdatasize[i],p = convdata[i]-1,tmp = 0;j<siz;++j) tmp += A[*(++p)]==LABELINDEX; B[sel] = tmp>siz*THR;}
#define THR .33 /*.4*/

/*mapping parameters temporary use*/
static real_T cx;
static real_T cy;
static real_T pa;
static real_T pb;

static int convdata[MAXDISTANCEPIX*MAXDISTANCEPIX*32/10][MAXCONVSIZE];
static int nconvdata,convdatasize[MAXDISTANCEPIX*MAXDISTANCEPIX*32/10];
static int convdataind[MAXDISTANCEPIX*MAXDISTANCEPIX*32/10];


static int circ(int* a,int x,int y,int n)
{
    int i,j,k;
    k = 0;
    for (i=-n;i<=n;i+=KERNELGRID)
        for (j=-n;j<=n;j+=KERNELGRID)
            if (i*i+j*j<=(n+.5)*(n+.5))
                if (KERNELNOISE) a[k++] = x+i+rand()%3-1+SIZEX*(y+j+rand()%3-1);
                else a[k++] = x+i+SIZEX*(y+j);
    return k;
}

static void prepareconv()
{
    int x,y,dist,n;
    nconvdata = 0;
    for (y=0;y<SIZEY;++y)
    {
        for (x=0;x<SIZEX;++x)
        {
            dist = sqrt((x-cx)*(x-cx)+(y-cy)*(y-cy));
            if (dist>MINDISTANCEPIX & dist<MAXDISTANCEPIX)
            {
                n = (int)(2*BALLRADIUS/(pa*tan(pb*dist))*dist*KERNELFAC+.5);
                if (n<MINKERNEL) n=MINKERNEL;
                if (n>MAXKERNEL) n=MAXKERNEL;
                convdatasize[nconvdata] = circ(convdata[nconvdata],x,y,n);
                convdataind[nconvdata] = x+y*SIZEX;
                ++nconvdata;
            }
        }
    }
}

static void mdlInitializeSizes(SimStruct *S)
{
    DECL_AND_INIT_DIMSINFO(inputDimsInfo);
    DECL_AND_INIT_DIMSINFO(outputDimsInfo);
    
    FILE *fid;
    double mc[9];
    
    /*PARAMETERS*/
    ssSetNumSFcnParams(S, 0);
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }
    
    fid=fopen("mc.dat","rb");
    if (fid!=NULL)
    {
        fread(mc,sizeof(double),9,fid);
        fclose(fid);
        
        cx = mc[0];
        cy = mc[1];
        pa = mc[2];
        pb = mc[3];
    }
    else
    {
        printf("could not find mc.dat!!!\n");
        return;
    }
    
    prepareconv();
    
    /*INPUT PORTS*/
    if (!ssSetNumInputPorts(S, 1)) return;
    
    inputDimsInfo.width = SIZEX;
    ssSetInputPortDimensionInfo(S, 0, &inputDimsInfo);
    ssSetInputPortMatrixDimensions(  S ,0, SIZEX, SIZEY);
    ssSetInputPortDataType(S, 0, SS_UINT8);
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortRequiredContiguous(S, 0, 1);
    
    /*OUTPUT PORTS*/
    if (!ssSetNumOutputPorts(S, 1)) return;
    
    outputDimsInfo.width = SIZEX;
    ssSetOutputPortDimensionInfo(S, 0, &outputDimsInfo);
    ssSetOutputPortMatrixDimensions( S ,0, SIZEX, SIZEY);
    ssSetOutputPortDataType(S, 0, SS_BOOLEAN);
}

static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}

static void mdlOutputs(SimStruct *S, int_T tid)
{
    /*inputs*/
    /*goalx contains [distance angle] of goal x*/
    const uint8_T *A  = (const uint8_T*) ssGetInputPortSignal(S,0);
    
    /*outputs*/
    bool *B  = (bool *)ssGetOutputPortRealSignal(S,0);
    
    int tmp,siz;
    int i,j,k,sel;
    int *p;
    
    /*distance dependent convolution*/
    for (i=NSTEP;i<nconvdata-NSTEP;i+=NSTEP)
    {
        FUNC(i); /*coarse steps*/
        if (B[sel]) 
        {
            for (k=i-NSTEP+1;k<i;++k) FUNC(k); /*fine steps*/
            for (k=i+NSTEP-1;k>i;--k) FUNC(k); /*fine steps*/
        }
    }
}

#define MDL_SET_INPUT_PORT_DIMENSION_INFO
static void mdlSetInputPortDimensionInfo(SimStruct        *S,
int_T            port,
const DimsInfo_T *dimsInfo)
{
    if(!ssSetInputPortDimensionInfo(S, port, dimsInfo)) return;
}

# define MDL_SET_OUTPUT_PORT_DIMENSION_INFO
static void mdlSetOutputPortDimensionInfo(SimStruct        *S,
int_T            port,
const DimsInfo_T *dimsInfo)
{
    if(!ssSetOutputPortDimensionInfo(S, port, dimsInfo)) return;
}

static void mdlTerminate(SimStruct *S)
{
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif


