function ccdata(action,hObject,handles)
%todo:
%-close button?
%-statustext or cancel button while addpolygon or indicate center?
%-disable other tools while addpolygon or indicate center?

%init
global toolopt;
caldir = fileparts(mfilename('fullpath'));
curdir = cd;
cd(caldir);
switch toolopt,
    case 'ball',
        clr='orange';
    case 'field',
        clr='green';
end

try
    switch action,
        %COLOR CALIBRATION
        case {'initcc','initmask'},
            %init color & mask calibration gui's
            set(hObject,'deletefcn',@closetool);
            initccgui(action,hObject,handles);
        case 'loadimage',
            reload=1;
            loadimage(hObject,handles,reload);
        case 'selectpolygon',
            selectpolygon(hObject,handles);
        case 'addcolor',
            handles = addcolorspace(hObject,handles,clr);
            guidata(hObject,handles)
        case 'remcolor',
            handles = remcolorspace(hObject,handles,clr);
            guidata(hObject,handles)
        case 'clearcolor',
            handles = clearcolorspace(hObject,handles,clr);
            guidata(hObject,handles)
        case 'save',
            savecalibration(hObject,handles);
        case 'load',
            reload=1;
            loadcalibration(hObject,handles,reload);
       
        %MASK CALIBRATION
        case 'center',
            centermask(hObject,handles);
        case 'addpolygon',
            addpolygonmask(hObject,handles);
        case 'clearmask',
%             guidata(hObject,handles);
            clearmask(hObject,handles);
        case {'minradius','maxradius'},
            plotsegm(hObject,handles);
        case {'addminradius','remminradius',...
            'addmaxradius','remmaxradius'},
            addremradii(action,hObject,handles);
        case 'savemask',
            savemask(hObject,handles);
        case 'loadmask',
            handles=loadmask(hObject,handles);
            
            %update calibration data
            if ~isempty(handles),
                switch toolopt,
                    case 'mask',
                        plotsegm(hObject,handles);
                    case 'heading',
                        plotheading(hObject,handles);
                end
            end

            %% HEADING CALIBRATION %%
        case 'heading',
            heading(hObject,handles);
        case 'switch',
            switchmaskheading(hObject,handles);
    end
catch
end

%finish
cd(curdir);

return;

%------------------------------------------------------
function switchmaskheading(hObject,handles)
%init
global toolopt;

%switch
if strcmp(toolopt,'heading'),
    toolopt='mask';
else toolopt='heading';
end

%update buttons
handles=maskheadingbuttons(hObject,handles,0);

%(re)load mask/heading
handles=loadmask(hObject,handles);

%update handles
guidata(hObject,handles);

%plot calibration data
switch toolopt,
    case 'mask',
        %discard heading
        set(handles.headingplot,'visible','off');

        %show labelimage axis
        plotsegm(hObject,handles);
        
    case 'heading',
        %discard labelimage axis
        cla(handles.labelimage);
        
        %show heading
        plotheading(hObject,handles);
end

return;

%------------------------------------------------------
function handles=maskheadingbuttons(hObject,handles,newmaskflag)

%init
global toolopt;

%disable / enable / rename buttons
switch toolopt,
    case 'heading',
        do1='on'; do2='off';
    case 'mask',
        do1='off'; do2='on';
end
set(handles.heading,'enable',do1);
set(handles.addpolygon,'enable',do2);
set(handles.clearmask,'enable',do2);

%enable circle calibration
if newmaskflag,
    do='on';
else do='off';
end
set(handles.remminradius,'enable',do);
set(handles.addminradius,'enable',do);
set(handles.remmaxradius,'enable',do);
set(handles.addmaxradius,'enable',do);
set(handles.smallradius,'enable',do);
set(handles.bigradius,'enable',do);
set(handles.center,'enable',do);
set(handles.radii,'enable',do);
set(handles.min,'enable',do);
set(handles.max,'enable',do);

%rename buttons / gui
if isfield(handles.data,'loadfile') && sum(handles.data.loadfile),
    set(gcf,'name',[toolopt,' calibration: ',handles.data.loadfile]);
else set(gcf,'name',[toolopt,' calibration']);
end
set(handles.savetext,'title',['Save / reload ',toolopt]);
set(handles.save,'string',['save ',toolopt]);
set(handles.reload,'string',['reload ',toolopt]);

return;

%------------------------------------------------------
function heading(hObject,handles)

%init
axes(handles.image)

%indicate robot heading
[robotheading_y,robotheading_x]=ginput(1);
handles.data.heading = [robotheading_x,robotheading_y];

%plot heading
plotheading(hObject,handles);

return;

%------------------------------------------------------
function plotheading(hObject,handles)

%init
axes(handles.image);

%plot heading
try
    set(handles.headingplot,'xdata',[handles.data.center(2),handles.data.heading(2)],...
        'ydata',[handles.data.center(1),handles.data.heading(1)]);
catch
    hold on;
    handles.headingplot=plot([handles.data.center(2),handles.data.heading(2)],...
        [handles.data.center(1),handles.data.heading(1)],'b','linewidth',2);
end

%finish
if isfield(handles.data,'loadfile') && sum(handles.data.loadfile),
    set(handles.headingplot,'visible','on');
else set(handles.headingplot,'visible','off');
end
guidata(hObject,handles);

return;

%------------------------------------------------------
function centermask(hObject,handles)

%indicate center
axes(handles.image)
[y,x,button]=ginput(1);

%update center if left mouse button is used
if button==1,
    handles.data.center=[x,y];
    plotsegm(hObject,handles);
end
return;

%------------------------------------------------------
function clearmask(hObject,handles)

%clear polygonmask
handles.data.polygonmask=ones(size(handles.data.polygonmask));

%enable circle calibration
handles=maskheadingbuttons(hObject,handles,1);

%update handles
guidata(hObject,handles);

%re-plot
plotsegm(hObject,handles);

return;

%------------------------------------------------------
function savemask(hObject,handles)

%init
global turtle toolopt;

%init
curdir=cd;
cd(['../../Calibration_data_turtle',num2str(turtle),'/']);

try
    switch toolopt,
        case 'mask',
            %compose mask
            if strcmp(get(handles.bigradius,'enable'),'on'),
                mask=maskcircles(handles);
                mask=uint8(255*mask.*handles.data.polygonmask);
            else
                mask=uint8(255*handles.data.polygonmask);
            end

            %save mask.dat
            pause(0.5)
            delete([cd '/mask.dat'])
            fid=fopen('mask.dat','wb');
            fwrite(fid,mask,'uint8');
            fclose(fid);

            %robot center
            robotcenter = [handles.data.center(1),handles.data.center(2)];

            %load current mc.dat
            %p(1:2): center of the robot used for self localisation
            %p(3:4): pixel to meter parameters
            %p(5:6): offset correction in m
            %p(7:8): real center of the robot determined by the mask center
            %p(9): angle of robot heading in image
            fid=fopen('mc.dat','rb');
            if fid == -1  %in case that the format is not correct or the calibration file does not exist
                params = [robotcenter,zeros(1,4),robotcenter,0];
            else
                params = fread(fid,9,'double');
                params = params(:)';

                %reset robotcenter
                params(7:8) = robotcenter;

                %calculate the offset correction
                center_robot_distance = sqrt((params(1)-params(7))^2+(params(2)-params(8))^2);
                params(5:6) = [-params(3)*tan(params(4)*center_robot_distance)*(params(2)-params(8))/center_robot_distance   params(3)*tan(params(4)*center_robot_distance)*(params(1)-params(7))/center_robot_distance];
            end
            fclose(fid);
            
        case 'heading',
            %load current mc.dat
            %p(1:2): center of the robot used for self localisation
            %p(3:4): pixel to meter parameters
            %p(5:6): offset correction in m
            %p(7:8): real center of the robot determined by the mask center
            %p(9): angle of robot heading in image
            fid=fopen('mc.dat','rb');
            if fid == -1  %in case that the format is not correct or the calibration file does not exist
                params = zeros(1,9);
            else
                params = fread(fid,9,'double');
                params = params(:)';
            end
            fclose(fid);
            
            %renew robot angle
            params(9) = atan2(handles.data.center(2)-handles.data.heading(2),...
                handles.data.center(1)-handles.data.heading(1));
    end

    %save updated mc.dat
    fclose('all');
    pause(.5)
    delete('mc.dat')
    fid=fopen('mc.dat','wb');
    fwrite(fid,params,'double');
    fclose(fid);

    %feedback
    ccc=get(hObject,'backgroundcolor');
    set(hObject,'backgroundcolor',[0 1 0]); drawnow
    pause(1.0)
    set(hObject,'backgroundcolor',ccc); drawnow
catch
    ccc=get(hObject,'backgroundcolor');
    set(hObject,'backgroundcolor',[1 0 0]); drawnow
    pause(1.0)
    set(hObject,'backgroundcolor',ccc); drawnow
end

%finish
cd(curdir);

return;

%------------------------------------------------------
function addremradii(action,hObject,handles)

switch action,
    case 'addminradius',
        set(handles.smallradius,'string',num2str(str2num(get(handles.smallradius,'string'))+1));
    case 'remminradius',
        set(handles.smallradius,'string',num2str(str2num(get(handles.smallradius,'string'))-1));
    case 'addmaxradius',
        set(handles.bigradius,'string',num2str(str2num(get(handles.bigradius,'string'))+1));
    case 'remmaxradius',
        set(handles.bigradius,'string',num2str(str2num(get(handles.bigradius,'string'))-1));
end

%finish
guidata(hObject,handles);

%update mask plot
plotsegm(hObject,handles);

return;

%------------------------------------------------------
function mcs=maskcircles(handles)

%get mask parameters
center=handles.data.center;
bigradius=str2num(get(handles.bigradius,'string'));
smallradius=str2num(get(handles.smallradius,'string'));

%redefine mask
[Y,X]=meshgrid(1:size(handles.data.polygonmask,2),1:size(handles.data.polygonmask,1));
mcs=1.0*((Y-center(2)).^2+(X-center(1)).^2<bigradius^2 & (Y-center(2)).^2+(X-center(1)).^2>smallradius^2);

return;
    
%------------------------------------------------------
function addpolygonmask(hObject,handles)

%select polygon
[Px,Py]=getpolygon(handles.image);
[y,x]=meshgrid(1:size(handles.data.im,2),1:size(handles.data.im,1));
newpolygon=~inpolygon(x,y,Px,Py);

%update mask data
handles.data.polygonmask = handles.data.polygonmask.*newpolygon;

%update mask plot
plotsegm(hObject,handles);

%finish
guidata(hObject,handles);

return;

%------------------------------------------------------
function handles=loadgreen(hObject,handles)

%init
global turtle
curdir=cd;
calibpath=['Calibration_data_turtle',num2str(turtle),'/'];

%field color calibration
try
    cd(['../../',calibpath]);
    filename='green.dat';
    fid=fopen(filename,'rb');
    handles.data.Cgreen(:)=fread(fid,prod(size(handles.data.Cgreen)),'bit8');
    fclose(fid);
    handles.data.green = find(handles.data.Cgreen==handles.data.colorsgreen);
    handles.green=1;
catch
    handles.green=0;
end

%finish
guidata(hObject,handles)
cd(curdir);
return;

%------------------------------------------------------
function handles=loadmask(hObject,handles)

%init
global turtle toolopt;
curdir=cd;
calibpath=['Calibration_data_turtle',num2str(turtle),'/'];

%load mask
try
    switch toolopt,
        case 'mask',
            cd(['../../',calibpath]);
            filename='mask.dat';
            fid=fopen(filename,'rb');
            A=fread(fid,[640 480],'*uint8');
            fclose(fid);
            handles.data.polygonmask=double(A/255.0);

            %load mirror calibration data
            fid=fopen('mc.dat','rb');
            mc=fread(fid,9,'double');
            fclose(fid);

            %mask (robot) center
            handles.data.center = mc(7:8);
            
            %disable circle calibration
            handles=maskheadingbuttons(hObject,handles,0);
            
        case 'heading',
            %load mirror calibration data
            cd(['../../',calibpath]);
            fid=fopen('mc.dat','rb');
            mc=fread(fid,9,'double');
            fclose(fid);

            %robot heading
            l=230;      %heading arrow length
            handles.data.heading(2) = (mc(8)-l*sin(mc(9)));
            handles.data.heading(1) = (mc(7)-l*cos(mc(9)));
            handles.data.center = mc(7:8);
    end

catch
    %feedback
    ccc=get(hObject,'backgroundcolor');
    set(hObject,'backgroundcolor',[1 0 0]); drawnow
    pause(1.0)
    set(hObject,'backgroundcolor',ccc); drawnow
    
    handles=[];
end

%finish
cd(curdir);

return;

%------------------------------------------------------
function handles=getobstacles(hObject,handles)
%init
global turtle
curdir=cd;

%OBSTACLE POINTS
data.fmdat = '../../fieldmap.dat';
data.caldat = ['../../Calibration_data_turtle',num2str(turtle),'/'];
data.images = '../capture_photo/';

%use MIC scheme to get obstacle points
try
    cd ../'Multi Image Calibration'/;
    get_input_data(data,fullfile(handles.data.loadpath,handles.data.loadfile));

    %MIC scheme input variables
    assignin('base','T',1);
    Y.time = 0;
    Y.signals.dimensions = [size(handles.data.im,1) size(handles.data.im,2)];
    U = Y;
    V = Y;
    Y.signals.values = handles.data.im(:,:,1);
    U.signals.values = handles.data.im(:,:,2);
    V.signals.values = handles.data.im(:,:,3);
    assignin('base','Y',Y);
    assignin('base','U',U);
    assignin('base','V',V);
    assignin('base','maxallowederror',get_global_par('WHITEPOINTS_ALLOWEDERROR'));

    sim('getlinepointspose');
    handles.obstaclepoints = obstaclepoints(:,:,end);

    delete('./*.dat');          %clean up MIC directory
    handles.doobstaclepoints = 1;
catch
    handles.doobstaclepoints = 0;
end
cd(curdir);
guidata(hObject,handles);
return;

%------------------------------------------------------
function savecalibration(hObject,handles)

%init
global turtle toolopt;
handles = getC(handles);

%init
curdir = cd;
cd ../..;

%save data
pathname=['Calibration_data_turtle',num2str(turtle),'/'];
if ~(isequal(pathname,0))
    cd(pathname)
    switch toolopt,
        case 'ball',
            fid=fopen('cc.dat','wb');
            fwrite(fid,handles.data.C(:),'uint8');
            fclose(fid);
        case 'field',
            fid=fopen('green.dat','wb');
            fwrite(fid,handles.data.Cgreen(:),'uint8');
            fclose(fid);
    end
end
cd(curdir)

%update succeeded feedback
ccc=get(hObject,'backgroundcolor');
set(hObject,'backgroundcolor',[0 1 0])
drawnow
pause(1)
set(hObject,'backgroundcolor',ccc)
drawnow
return;

%------------------------------------------------------
function handles=loadcolors(hObject,handles)

%init
global turtle
curdir=cd;
calibpath=['Calibration_data_turtle',num2str(turtle),'/'];

%field color calibration
try
    cd(['../../',calibpath]);
    filename='cc.dat';
    fid=fopen(filename,'rb');
    handles.data.C(:)=fread(fid,prod(size(handles.data.C)),'bit8');
    fclose(fid);
    handles.data.orange = find(handles.data.C==handles.data.colorsorange);
    %         handles.data.green = find(handles.data.C==handles.data.colorsgreen);
    handles.data.white = find(handles.data.C==handles.data.colorswhite);
    handles.data.black = find(handles.data.C==handles.data.colorsblack);
    handles.data.blue = find(handles.data.C==handles.data.colorsblue);
    handles.data.yellow = find(handles.data.C==handles.data.colorsyellow);
catch
    error('could not load color calibration!');
end

%finish
cd(curdir);
guidata(hObject,handles)
return;
        
%------------------------------------------------------
function varargout=loadcalibration(hObject,handles,reload)

%init
global toolopt;

%color calibration data
switch toolopt,
    case 'ball',
        handles=loadcolors(hObject,handles);
    case 'field',
        handles=loadgreen(hObject,handles);
end

%finish
if reload,
    plotrgb(handles)
    plotsegm(hObject,handles)
    guidata(hObject,handles)
end

if nargout,
    varargout{1}=handles;
end

return;

%------------------------------------------------------
function handles = addcolorspace(hObject,handles,col)

%get RGB data of selected points
RGB=[handles.data.R handles.data.G handles.data.B];

%make tight mesh
[r,g,b]=meshgrid(min(handles.data.R):max(handles.data.R),...
        min(handles.data.G):max(handles.data.G),...
        min(handles.data.B):max(handles.data.B));
r=r(:);g=g(:); b=b(:);

%compute convex hull
in = inhull(double([r g b]),double(RGB),[],1);
r=int32(r(in));
g=int32(g(in));
b=int32(b(in));

%determine indices in colormatrix
ind=sub2ind(size(handles.data.C),r+1,g+1,b+1);
eval(['handles.data.' col '= union(' 'handles.data.' col ',ind);'])
% ['handles.data.',col,'=handles.data.',col,',ind

% if ~isequal(col,'orange'), eval(['handles.data.' col '=setdiff(handles.data.' col ',handles.data.orange);']), end
% if ~isequal(col,'green'), eval(['handles.data.' col '=setdiff(handles.data.' col ',handles.data.green);']), end
% if ~isequal(col,'white'), eval(['handles.data.' col '=setdiff(handles.data.' col ',handles.data.white);']), end
% if ~isequal(col,'black'), eval(['handles.data.' col '=setdiff(handles.data.' col ',handles.data.black);']), end
% if ~isequal(col,'blue'), eval(['handles.data.' col '=setdiff(handles.data.' col ',handles.data.blue);']), end
% if ~isequal(col,'yellow'), eval(['handles.data.' col '=setdiff(handles.data.' col ',handles.data.yellow);']), end
   
handles=getC(handles);
plotrgb(handles)
plotsegm(hObject,handles)

return;

%------------------------------------------------------
function handles = remcolorspace(hObject,handles,col)
RGB=[handles.data.R handles.data.G handles.data.B];

%make tight mesh
[r,g,b]=meshgrid(min(handles.data.R):max(handles.data.R),...
        min(handles.data.G):max(handles.data.G),...
        min(handles.data.B):max(handles.data.B));
r=r(:);g=g(:); b=b(:);

in = inhull(double([r g b]),double(RGB),[],2);
r=int32(r(in));
g=int32(g(in));
b=int32(b(in));
ind=sub2ind(size(handles.data.C),r+1,g+1,b+1);

eval(['handles.data.' col '= setdiff(handles.data.' col ',ind);'])

handles=getC(handles);
plotrgb(handles)
plotsegm(hObject,handles)

%------------------------------------------------------
function handles=clearcolorspace(hObject,handles,col)

eval(['handles.data.' col '=[];'])

handles=getC(handles);
plotrgb(handles)
plotsegm(hObject,handles)

return;


%------------------------------------------------------
function selectpolygon(hObject,handles)

try delete(handles.data.h_xy), catch end
try delete(handles.data.h_RGB), catch end

% set(gcf,'keypressfcn',@keypressfcn);
[Px,Py]=getpolygon(handles.image);
% set(gcf,'',@keypressfcn);

%get points inside the polygon
[x,y]=meshgrid(min(Px):max(Px),min(Py):max(Py));
in=inpolygon(x,y,Px,Py);
handles.data.x=int32(round(x(in)));
handles.data.y=int32(round(y(in)));

%plot points
axes(handles.image)
handles.data.h_xy = plot(handles.data.y,handles.data.x,'k.');

%get RGB values of these points
ind=sub2ind(size(handles.data.D),handles.data.x,handles.data.y);
handles.data.R=handles.data.D(ind);
handles.data.G=handles.data.D(ind+prod(size(handles.data.D))/3);
handles.data.B=handles.data.D(ind+prod(size(handles.data.D))/3*2);

%plot points
axes(handles.colorspace)
handles.data.h_RGB = plot3(handles.data.R*handles.data.ds,handles.data.G*handles.data.ds,handles.data.B*handles.data.ds,'k.');
axis([0 255 0 255 0 255])

guidata(hObject,handles)
return;

%------------------------------------------------------
function varargout=loadimage(hObject,handles,reload)

%init
global turtle toolopt;
curdir=cd;
cd ../capture_photo;

[loadfile,loadpath]=uigetfile(['turtle',num2str(turtle),'*.png'],'Import image');

handles.data.loadfile=loadfile;
handles.data.loadpath=loadpath;
if loadfile
    cd(loadpath);
    handles.data.im=uint8(imread(loadfile));
    handles.data.D=(handles.data.im-1)/handles.data.ds;

    axes(handles.image);
    cla
    imshow(ycbcr2rgb(handles.data.im))
    guidata(hObject, handles);
    cd(curdir);
    
    %get obstacles
    handles=getobstacles(hObject,handles);
    guidata(hObject,handles);
    
    %plot data
    if reload,
        switch toolopt,
            case 'heading',
                plotheading(hObject,handles);
            otherwise,
                plotsegm(hObject,handles);
        end
    end
    
    %set gui name
    set(gcf,'Name',[toolopt,' calibration: ',handles.data.loadfile]);
    
end

if nargout,
    varargout{1}=handles;
end
cd(curdir);
return;

%------------------------------------------------------
function initccgui(type,hObject,handles)

%init
global toolopt turtle;
curdir=cd;
handles.output = hObject;

%check existence of global variables
if isempty(turtle) || isempty(str2double(turtle)),
    str={};
    for i1=1:get_global_par('MAX_TURTLES'),
        str{i1}=['turtle ',int2str(i1)];
    end
    [turtle,v] = listdlg('PromptString','Select a robot:',...
        'SelectionMode','single','name','Robot selection',...
        'ListString',str,'listsize',[100 get_global_par('MAX_TURTLES')*18]);
    if ~v,
        close(hObject);
        return;
    end
end
if strcmp(type,'initcc') && isempty(toolopt),
    answer = questdlg('What do you want to calibrate?', ...
        'Calibration options', ...
        'Ball color', 'Field color','Ball color');
    switch answer,
        case 'Ball color',
            toolopt='ball';
        case 'Field color',
            toolopt='field';
        otherwise
            close(hObject);
            return;
    end
end
if strcmp(type,'initmask') && isempty(toolopt),
    answer = questdlg('What do you want to calibrate?', ...
        'Calibration options', ...
        'Robot mask', 'Robot heading','Robot mask');
    switch answer,
        case 'Robot mask',
            toolopt='mask';
        case 'Robot heading',
            toolopt='heading';
        otherwise
            close(hObject);
            return;
    end
end

%set turtle name
set(handles.turtlename,'title',['Turtle ',num2str(turtle)]);

%set gui name
set(hObject,'Name',[toolopt,' calibration']);

%downsampling
handles.data.ds = uint8(2);

%initialize color and green colorspaces
handles.data.C = zeros(256/handles.data.ds,256/handles.data.ds,256/handles.data.ds,'uint8');
handles.data.Cgreen = zeros(256/handles.data.ds,256/handles.data.ds,256/handles.data.ds,'uint8');

%define colors
handles.data.colororange=[1 .5 0];
handles.data.colorgreen=[0 .5 0];
handles.data.colorwhite=[1 1 1];
handles.data.colorblack=[.5 .5 .5];
handles.data.colorblue=[0 0 1];
handles.data.coloryellow=[1 1 0];

%define colorindices
handles.data.colorsorange=1;
handles.data.colorsgreen=2;
handles.data.colorswhite=4;
handles.data.colorsblack=8;
handles.data.colorsblue=16;
handles.data.colorsyellow=32;
handles.data.colors=[0 0 0;handles.data.colororange;...
    handles.data.colorgreen;zeros(1,3);...
    handles.data.colorwhite;zeros(3,3);...
    handles.data.colorblack;zeros(7,3);...
    handles.data.colorblue;zeros(15,3);...
    handles.data.coloryellow];

%init mask parameters
handles.data.polygonmask = ones(640,480);
handles.data.center = [0 0];
handles.data.heading = [0 0];

%set toolopt specific settings
if strcmp(type,'initcc'),
    switch toolopt,
        case 'ball',
            set(handles.calibrationpanel,'title','Ball color calibration');
            set(handles.selectcolor,'backgroundcolor',handles.data.colororange,'string','ball color');
        case 'field',
            set(handles.calibrationpanel,'title','Field color calibration');
            set(handles.selectcolor,'backgroundcolor',handles.data.colorgreen,'string','field color');
    end
else handles=maskheadingbuttons(hObject,handles,0);
end

%load colorcalibration
handles=loadcolors(hObject,handles);

%load mask
handles=loadmask(hObject,handles);

%load green
handles=loadgreen(hObject,handles);

%set image axis properties
axes(handles.image);
handles.data.im = zeros(640,480,3,'uint8');
imshow(handles.data.im);

%load image
handles=loadimage(hObject,handles,0);

%set labelimage axis properties
axis(handles.labelimage,'off');

%set colorspace axis properties
if strcmp(type,'initcc'),
    axes(handles.colorspace);
    hold on;
    axis([0 255 0 255 0 255]);
    view(90,0);
    set(gca,'color',[.7 .7 .7],'box','on',...
        'xticklabel',[],'yticklabel',[],'zticklabel',[]);
end

%get obstacles
handles=getobstacles(hObject,handles);

%update handles
guidata(hObject, handles);

%plot calibration colorspace
if strcmp(type,'initcc'),
    plotrgb(handles)
end

%plot calibration data
switch toolopt,
    case {'ball','field'},
        plotsegm(hObject,handles);
    case 'mask',
        plotsegm(hObject,handles);
    case 'heading',
        plotheading(hObject,handles);
end

%finish
cd(curdir);

return;

%------------------------------------------------------
function plotrgb(handles)

%init
global toolopt;
axes(handles.colorspace)
cla

switch toolopt,
    case 'ball',
        c=handles.data.colororange;
        [r,g,b] = ind2sub(size(handles.data.C),handles.data.orange);
        plot3(r*double(handles.data.ds),g*double(handles.data.ds),b*double(handles.data.ds),'.','color',c);
    case 'field',
        c=handles.data.colorgreen;
        [r,g,b] = ind2sub(size(handles.data.C),handles.data.green);
        plot3(r*double(handles.data.ds),g*double(handles.data.ds),b*double(handles.data.ds),'.','color',c);
end

% c=handles.data.colorwhite;
% [r,g,b] = ind2sub(size(handles.data.C),handles.data.white);
% plot3(r*double(handles.data.ds),g*double(handles.data.ds),b*double(handles.data.ds),'.','color',c);
% 
% c=handles.data.colorblack;
% [r,g,b] = ind2sub(size(handles.data.C),handles.data.black);
% plot3(r*double(handles.data.ds),g*double(handles.data.ds),b*double(handles.data.ds),'.','color',c);
% 
% c=handles.data.colorblue;
% [r,g,b] = ind2sub(size(handles.data.C),handles.data.blue);
% plot3(r*double(handles.data.ds),g*double(handles.data.ds),b*double(handles.data.ds),'.','color',c);
% 
% c=handles.data.coloryellow;
% [r,g,b] = ind2sub(size(handles.data.C),handles.data.yellow);
% plot3(r*double(handles.data.ds),g*double(handles.data.ds),b*double(handles.data.ds),'.','color',c);

return;


%------------------------------------------------------
function plotsegm(hObject,handles)

%init
global toolopt;
axes(handles.image)
try
    handles.data.D;
catch
    return;
end
k1=int32(1+handles.data.D(1:640*480));
k2=int32(1+handles.data.D((1:640*480)+640*480));
k3=int32(1+handles.data.D((1:640*480)+640*480*2));

%get ball color
indC=sub2ind(size(handles.data.C),k1,k2,k3);
ball=handles.data.C(indC);
ball=reshape(ball,[640 480]);

%get field color
indG=sub2ind(size(handles.data.Cgreen),k1,k2,k3);
field=handles.data.Cgreen(indG);
field=reshape(field,[640 480]);

%combine ball (orange) and field (green)
clrmap = handles.data.colorsgreen*uint8(field~=0);
clrmap(ball==handles.data.colorsorange)=handles.data.colorsorange;

%clean up
try delete(handles.data.h_xy), catch end
try delete(handles.data.h_RGB), catch end

%add obstacle points
if handles.doobstaclepoints,
    ind = find(handles.obstaclepoints==1);
    z = [-2:2 -640*2:640:2*640 641 639 -641 -639];
    for i=1:length(z)
        clrmap(ind+z(i))=3;  %possible obstacles (blue)
    end
    ind = find(handles.obstaclepoints==5);
    for i=1:length(z)
        clrmap(ind+z(i))=4;  %detected obstacles (white)
    end
end

%add mask
if strcmp(toolopt,'mask') && strcmp(get(handles.bigradius,'enable'),'on'),
    mask=maskcircles(handles);
    clrmap=clrmap.*uint8(mask);
end
clrmap=clrmap.*uint8(handles.data.polygonmask);

%plot calibration data
axes(handles.labelimage); cla;
imshow(clrmap);
clmap_max=max(clrmap(:));
caxis([0 clmap_max]);
cm = [0 0 0;handles.data.colororange;handles.data.colorgreen;...
    handles.data.colorblue;handles.data.colorwhite];
colormap(cm(1:clmap_max+1,:))

%finish
guidata(hObject,handles);

return;

%------------------------------------------------------
function handles=getC(handles)

%init
global toolopt;

switch toolopt,
    case 'ball',
        handles.data.C(:,:,:)=0;
        handles.data.C(handles.data.orange)=handles.data.colorsorange;
%         handles.data.C(handles.data.green)=handles.data.colorsgreen;
%         handles.data.C(handles.data.white)=handles.data.colorswhite;
%         handles.data.C(handles.data.black)=handles.data.colorsblack;
%         handles.data.C(handles.data.blue)=handles.data.colorsblue;
%         handles.data.C(handles.data.yellow)=handles.data.colorsyellow;
    case 'field',
        handles.data.Cgreen(:,:,:)=0;
        handles.data.Cgreen(handles.data.green)=handles.data.colorsgreen;
end

return;
