#define S_FUNCTION_NAME compasscheck
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"
#include "math.h"
#include "../../Global_par/constants.h"

#define COPYPOSE {for(i=0;i<3;++i) posevisionflipped[i]=posevision[i];\
                 posefoundflipped[0]=posefound[0];}
#define FLIPPOSE {for(i=0;i<2;++i) posevisionflipped[i]=-posevision[i];\
                 posevisionflipped[2]=posevision[2]+PI;\
                 if (posevisionflipped[2]>=PI) posevisionflipped[2]-=2*PI;\
                 posefoundflipped[0]=posefound[0];}
static real_T heading_bluegoal;

static void mdlInitializeSizes(SimStruct *S)
{
    DECL_AND_INIT_DIMSINFO(inputDimsInfo);
    FILE *fid;
 
    fid=fopen("compass.dat","rb");
    if (fid!=NULL)
    {
        fread(&heading_bluegoal,sizeof(double),1,fid);
        fclose(fid);
    }
    else
    {
        printf("could not find compass.dat!!!");
        return;
    }
    
    /*PARAMETERS*/
    ssSetNumSFcnParams(S, 0);
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }
    
    /*INPUT PORTS*/
    if (!ssSetNumInputPorts(S, 4)) return;
    
    ssSetInputPortWidth(S, 0, 3);
    ssSetInputPortDataType(S, 0, SS_DOUBLE);
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortRequiredContiguous(S, 0, 1);
    
    ssSetInputPortWidth(S, 1, 1);
    ssSetInputPortDataType(S, 1, SS_INT32);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
    ssSetInputPortRequiredContiguous(S, 1, 1);
    
    ssSetInputPortWidth(S, 2, 1);
    ssSetInputPortDataType(S, 2, SS_DOUBLE);
    ssSetInputPortDirectFeedThrough(S, 2, 1);
    ssSetInputPortRequiredContiguous(S, 2, 1);
    
    ssSetInputPortWidth(S, 3, 1);
    ssSetInputPortDataType(S, 3, SS_INT32);
    ssSetInputPortDirectFeedThrough(S, 3, 1);
    ssSetInputPortRequiredContiguous(S, 3, 1);
    
    /*OUTPUT PORTS*/
    if (!ssSetNumOutputPorts(S, 3)) return;
    
    ssSetOutputPortWidth(S, 0, 3);
    ssSetOutputPortDataType(S, 0, SS_DOUBLE);
    
    ssSetOutputPortWidth(S, 1, 1);
    ssSetOutputPortDataType(S, 1, SS_INT32);
    
    ssSetOutputPortWidth(S, 2, 1);
    ssSetOutputPortDataType(S, 2, SS_DOUBLE);    
}

static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}

static void mdlOutputs(SimStruct *S, int_T tid)
{
    /*inputs*/
    const real_T *posevision  = (const real_T*) ssGetInputPortSignal(S,0);
    const int_T *posefound  = (const int_T*) ssGetInputPortSignal(S,1);
    const real_T *compassangle  = (const real_T*) ssGetInputPortSignal(S,2);
    const int_T *blueishomegoal  = (const int_T*) ssGetInputPortSignal(S,3);

    /*outputs*/
    real_T *posevisionflipped = (real_T*) ssGetOutputPortSignal(S,0);
    int_T *posefoundflipped  = (int_T*) ssGetOutputPortSignal(S,1);
    real_T *headingbluegoal_out = (real_T*) ssGetOutputPortSignal(S,2);
    
    real_T diffangle, bluegoalangle;
    int_T i;
    
    /*location of blue goal*/
    bluegoalangle = compassangle[0] - heading_bluegoal;
    
    /*get angle difference between pose and compass*/
    diffangle = bluegoalangle+posevision[2]*180/PI;
    
    /*printf("posefound %d, diffangle: %f, posevision: %f compassangle: %f, heading_bluegoal %f ",posefound[0],diffangle,posevision[2]*180/M_PI,compassangle[0], heading_bluegoal);*/
    
    if (posefound[0]==1)
    {
        /* 0<=diffangle<2*PI */
        diffangle = fmod(diffangle,360);
        if (diffangle<0) diffangle+=360;
        
        /*estimated pose with yellow goal as home goal*/
        if (diffangle<COMPASSANGLETHRESHOLD || diffangle>360-COMPASSANGLETHRESHOLD)
        {
            if (blueishomegoal[0]) FLIPPOSE
            else COPYPOSE
        }
        /*estimated pose with blue goal as home goal*/
        else if (diffangle>180-COMPASSANGLETHRESHOLD && diffangle<180+COMPASSANGLETHRESHOLD)
        {
            if (blueishomegoal[0]) COPYPOSE
            else FLIPPOSE
        }
        else
        {
            COPYPOSE
            posefoundflipped[0] = 0;
        }
    }
    else
    {
        COPYPOSE
        posefoundflipped[0] = 0;
    }
    headingbluegoal_out[0] = heading_bluegoal;
    
    posevisionflipped[2] = fmod(posevisionflipped[2]+M_PI,2*M_PI)-M_PI;
    if (posevisionflipped[2]<-M_PI) posevisionflipped[2]+=2*M_PI;
}

#undef MDL_INITIALIZE_CONDITIONS   /* Change to #undef to remove function */
#undef MDL_UPDATE  /* Change to #undef to remove function */
#undef MDL_DERIVATIVES  /* Change to #undef to remove function */

static void mdlTerminate(SimStruct *S)
{
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
