#include "mex.h"
#include "math.h"

double getmin(double x, double y)
{
    if (x<y) return x;
    else return y;
}

double getmax(double x, double y)
{
    if (x>y) return x;
    else return y;
}

void mexFunction( int nlhs, mxArray *plhs[],int nrhs, const mxArray *prhs[])
{
    double *pp,*fieldmap,*linepoints,*x,*y,*J,*x1,*x2,p[7];
    double fieldwidth,fieldlength,resolution,borderwidth;
    double X,Y,phi,sinphi,cosphi,sinp6,cosp6,minx,maxx,miny,maxy,w,h,r;
    int n,i,fx,fy,xx,yy;
    
    /*get input variables*/
    pp = mxGetPr(prhs[0]);
    fieldmap = mxGetPr(prhs[1]);
    fieldwidth = *mxGetPr(prhs[2]);
    fieldlength = *mxGetPr(prhs[3]);
    resolution = *mxGetPr(prhs[4]);
    borderwidth = *mxGetPr(prhs[5]);
    linepoints = mxGetPr(prhs[6]);
    x1 = mxGetPr(prhs[7]);
    x2 = mxGetPr(prhs[8]);
    n = mxGetM(prhs[6]);
    fx = mxGetM(prhs[1]);
    fy = mxGetN(prhs[1]);
    
    /*get output variable*/
    plhs[0] = mxCreateDoubleMatrix(1,1,mxREAL);
    J = mxGetPr(plhs[0]);
    
    for (i=0;i<7;++i)
    {
        if (pp[i]<0 || pp[i]>1) {J[0]=1e10; return;}
        p[i] = pp[i]*(x2[i]-x1[i])+x1[i];
    }
    
    if (n<=0) {J[0]=1e10; return;};
    
    /*aux. variables*/
    x = (double *)mxMalloc(sizeof(double)*n);
    y = (double *)mxMalloc(sizeof(double)*n);
    
    /*determine global coordinates of linepoints*/
    sinp6=sin(p[6]);
    cosp6=cos(p[6]);
    for(i=0;i<n;++i)
    {
        X = linepoints[n+i]-p[1];
        Y = p[0] - linepoints[i];
        phi = atan2(-X,Y);
        sinphi = sin(phi);
        cosphi = cos(phi);
        r = p[2]*tan(p[3]*sqrt(X*X+Y*Y));
        x[i] = p[4]-r*(sinphi*cosp6+cosphi*sinp6);
        y[i] = p[5]-r*(sinphi*sinp6-cosphi*cosp6);
    }
    
    /*get min and max of x and y*/
    minx = x[0];
    maxx = x[0];
    miny = y[0];
    maxy = y[0];
    for(i=1;i<n;++i)
    {
        minx = getmin(minx,x[i]);
        maxx = getmax(maxx,x[i]);
        miny = getmin(miny,y[i]);
        maxy = getmax(maxy,y[i]);
    }
    
    /*determine costfcn value*/
    J[0] = 0;
    if (!(maxx-minx>fieldwidth*.3 && maxy-miny>fieldlength*.3))
    {
        J[0] = 1e10;
    }
    else
    {
        w = (fieldwidth*.5+borderwidth)*resolution;
        h = (fieldlength*.5+borderwidth)*resolution;
        for(i=0;i<n;++i)
        {
            xx = (int)(x[i]*resolution+w+.5);
            yy = (int)(y[i]*resolution+h+.5);
            if (xx<0 || xx>=fx || yy<0 || yy>=fy) J[0]+= 255;
            else J[0] += fieldmap[xx+fx*yy];
        }
    }
    J[0]=J[0]/256/n;
}
