#define S_FUNCTION_NAME findball3D
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"
#include "math.h"
#include "../../Global_par/constants.h"

#define MAXBALLS 10
#define MINBALLSIZE 2
#define NPHI 16
#define CHECKRADIUS 1.7
#define CHECKCENTER 0.3
#define CDIFF2 (10*10)
#define CONTRASTFRAC .4
#define INVRADIUSMAPPING(r) (8.5*exp(-.18*r)) /*((1.13-log(r)/2.302585092994046)/.103*0.649)*/
#define BALLDISTCORRECTION 0.83
#define THRFLOORAIRU .4 /*m*/
#define THRFLOORAIRL -1 /*m*/
#define HEIGHTDIST 10
#define ROBOTHEIGHT 0.7

static double mc[9],camx,camy,par_a,par_b,correctionx,correctiony,correctionphi;
static double cosphi[NPHI],sinphi[NPHI],closestdistance;

static void resetoutput(real_T* ball,real_T* ballconf,real_T* allballs,real_T* allballsconf,int_T* nballs, const real_T* pose)
{
    int i;
    
    ball[0] = pose[0];
    ball[1] = pose[1];
    ball[2] = 0;
    ballconf[0] = 0;
    for (i=0;i<MAXBALLS;++i)
    {
        allballs[3*i]=pose[0];
        allballs[3*i+1]=pose[1];
        allballs[3*i+2]=0;
        allballsconf[i]=0;
    }
    nballs[0] = 0;
}

static int checkcontrast(int_T ballsize, const real_T* centroid,const uint8_T* U,const uint8_T* V)
{
    double r,Ucc,Vcc,UVsum[NPHI];
    int x,y,xb,xe,yb,ye,cnt,i;
    
    /*check bounds*/
    r = ballsize*.5;
    if ((int)(centroid[0]-r*CHECKRADIUS+.5)<=0 || 
        (int)(centroid[0]+r*CHECKRADIUS+.5)>=SIZEX-1 ||
        (int)(centroid[1]-r*CHECKRADIUS+.5)<=0 || 
        (int)(centroid[1]+r*CHECKRADIUS+.5)>=SIZEY-1) return 1;

    /*get center color*/
    xb = (int)(centroid[0]-r*CHECKCENTER+.5);
    xe = (int)(centroid[0]+r*CHECKCENTER+.5);
    yb = (int)(centroid[1]-r*CHECKCENTER+.5);
    ye = (int)(centroid[1]+r*CHECKCENTER+.5);
    Ucc = 0;
    Vcc = 0;
    for (x=xb;x<xe+1;++x)
    {
        for (y=yb;y<ye+1;++y)
        {
            Ucc+=U[x+y*640];
            Vcc+=V[x+y*640];
        }
    }
    Ucc /= (ye+1-yb)*(xe+1-xb);
    Vcc /= (ye+1-yb)*(xe+1-xb);
    
    /*get color contrast around ball*/
    cnt = 0;
    for (i=0;i<NPHI;++i)
    {
        x = (int)(centroid[0]+r*CHECKRADIUS*cosphi[i]+.5);
        y = (int)(centroid[1]+r*CHECKRADIUS*sinphi[i]+.5);
        UVsum[i] = (U[x+y*640]-Ucc)*(U[x+y*640]-Ucc)+(V[x+y*640]-Vcc)*(V[x+y*640]-Vcc);
        /*printf("%f\n",UVsum[i]);*/
        if (UVsum[i]<CDIFF2) ++cnt;
    }
    
    /*test color*/
    /*printf("cnt: %d < %f, radius: %f\n",cnt,CONTRASTFRAC*NPHI,r);*/
    if (cnt>CONTRASTFRAC*NPHI) return 1;
    else return 0;
}

static void get_ball_global(real_T* ball_global,const real_T* pose,const real_T* centroid,real_T d,real_T h)
{
    double a, ball_local[3],balx,baly;
    a = atan2(-(centroid[1]-camy),-(centroid[0]-camx));
    balx = -d*sin(a)-correctionx;
    baly = d*cos(a)-correctiony;
    ball_local[0] = cos(correctionphi)*balx+sin(correctionphi)*baly;
    ball_local[1] = -sin(correctionphi)*balx+cos(correctionphi)*baly;
    ball_local[2] = h;
    ball_global[0] = cos(pose[2])*ball_local[0] - sin(pose[2])*ball_local[1] + pose[0];
    ball_global[1] = sin(pose[2])*ball_local[0] + cos(pose[2])*ball_local[1] + pose[1];
    ball_global[2] = ball_local[2];
}

static void ball2output(int_T ballsize, const real_T* centroid,real_T* ball,real_T* ballconf,real_T* allballs,real_T* allballsconf,int_T* nballs,const real_T* pose)
{
    double angle,maxviewangle,m,d,h,mb,ball_global[3];
    
    /*distance of ball, assuming ball at ground (projected center)*/
    angle = par_b*sqrt((camx-centroid[0])*(camx-centroid[0])+(camy-centroid[1])*(camy-centroid[1]));
    maxviewangle = atan(20000/BALLDISTCORRECTION/par_a);
    if (angle>maxviewangle) angle=maxviewangle;
    m = BALLDISTCORRECTION*par_a*tan(angle);
    
    /*check if projection of ball is inside field*/
    get_ball_global(ball_global,pose,centroid,m,0);
   /* printf("ball global: %f %f\n",ball_global[0],ball_global[1]);*/
    if (fabs(ball_global[0])>FIELDWIDTH*.5+BORDERMARGIN_SMALL || fabs(ball_global[1])>FIELDLENGTH*.5+BORDERMARGIN_SMALL) return;
    
    /*nominal size of ball at that distance*/
    d = INVRADIUSMAPPING(ballsize*.5);
    
    /*estimated distance based on ball size*/
    h = ROBOTHEIGHT*(1-sqrt(d*d+ROBOTHEIGHT*ROBOTHEIGHT)/sqrt(m*m+ROBOTHEIGHT*ROBOTHEIGHT));
    
    /*height of ball*/
    /*printf("ball height: %f\n",h);*/
    if (h<=THRFLOORAIRU && h>=THRFLOORAIRL)
    {
        d = m; /*ground*/
        h = 0;
    }
    else if (h>THRFLOORAIRU)
    {
        if (m>=HEIGHTDIST) 
        {
            h = ROBOTHEIGHT-d*cos(angle);
            d = d*sin(angle);
        }
    }
    else return;
    
        /*check if ball lies in field*/
    get_ball_global(ball_global,pose,centroid,d,h);
    if (fabs(ball_global[0])<FIELDWIDTH*.5+BORDERMARGIN_SMALL && fabs(ball_global[1])<FIELDLENGTH*.5+BORDERMARGIN_SMALL)
    {
        /*add to allballs*/
        allballsconf[nballs[0]] = 1;
        allballs[nballs[0]*3] = ball_global[0];
        allballs[nballs[0]*3+1] = ball_global[1];
        allballs[nballs[0]*3+2] = ball_global[2];
        nballs[0]++;
     /*   printf("add ball: %f %f %f\n",ball_global[0],ball_global[1],ball_global[2]);*/
        
        /*add to nearest ball*/
        if (ballconf[0]==0 || (ballconf[0]==1 && d<closestdistance))
        {
            closestdistance = d;
            ballconf[0] = 1;
            ball[0] = ball_global[0];
            ball[1] = ball_global[1];
            ball[2] = ball_global[2];
        }
    }
}

static void mdlInitializeSizes(SimStruct *S)
{
    DECL_AND_INIT_DIMSINFO(inputDimsInfo);
    DECL_AND_INIT_DIMSINFO(outputDimsInfo);

   /*PARAMETERS*/
    FILE *fid;
    int i;
    
    ssSetNumSFcnParams(S, 0);
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }
    
    fid=fopen("mc.dat","rb");
    if (fid!=NULL)
    {
        fread(mc,sizeof(double),9,fid);
        fclose(fid);
        camx = mc[0];
        camy = mc[1];
        par_a = mc[2];
        par_b = mc[3];
        correctionx = mc[4];
        correctiony = mc[5];
        correctionphi = mc[8];
    }
    else 
    {
        printf("could not find mc.dat!!!");
        return;
    }
    
    for (i=0;i<NPHI;++i)
    {
        cosphi[i] = cos(M_PI*i/NPHI);
        sinphi[i] = sin(M_PI*i/NPHI);
    }
    
    /*INPUT PORTS*/
    if (!ssSetNumInputPorts(S, 7)) return;
    
    if(!ssSetInputPortDimensionInfo( S, 0, DYNAMIC_DIMENSION)) return;
    ssSetInputPortDataType(S, 0, SS_DOUBLE);
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortRequiredContiguous(S, 0, 1);
    
    if(!ssSetInputPortDimensionInfo( S, 1, DYNAMIC_DIMENSION)) return;
    ssSetInputPortDataType(S, 1, SS_INT32);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
    ssSetInputPortRequiredContiguous(S, 1, 1);
     
    ssSetInputPortWidth(S, 2, 1);
    ssSetInputPortDataType(S, 2, SS_UINT16);
    ssSetInputPortDirectFeedThrough(S, 2, 1);
    ssSetInputPortRequiredContiguous(S, 2, 1);
    
    inputDimsInfo.width = SIZEX;
    ssSetInputPortDimensionInfo(S, 3, &inputDimsInfo);
    ssSetInputPortMatrixDimensions(  S ,3, SIZEX, SIZEY);
    ssSetInputPortDataType(S, 3, SS_UINT8);
    ssSetInputPortDirectFeedThrough(S, 3, 1);
    ssSetInputPortRequiredContiguous(S, 3, 1);
    
    inputDimsInfo.width = SIZEX;
    ssSetInputPortDimensionInfo(S, 4, &inputDimsInfo);
    ssSetInputPortMatrixDimensions(  S ,4, SIZEX, SIZEY);
    ssSetInputPortDataType(S, 4, SS_UINT8);
    ssSetInputPortDirectFeedThrough(S, 4, 1);
    ssSetInputPortRequiredContiguous(S, 4, 1);
    
    inputDimsInfo.width = SIZEX;
    ssSetInputPortDimensionInfo(S, 5, &inputDimsInfo);
    ssSetInputPortMatrixDimensions(  S ,5, SIZEX, SIZEY);
    ssSetInputPortDataType(S, 5, SS_UINT8);
    ssSetInputPortDirectFeedThrough(S, 5, 1);
    ssSetInputPortRequiredContiguous(S, 5, 1);
     
    ssSetInputPortWidth(S, 6, 3);
    ssSetInputPortDataType(S, 6, SS_DOUBLE);
    ssSetInputPortDirectFeedThrough(S, 6, 1);
    ssSetInputPortRequiredContiguous(S, 6, 1);
   
    /*OUTPUT PORTS*/
    if (!ssSetNumOutputPorts(S, 5)) return;
    
    ssSetOutputPortWidth(S, 0, 3);
    ssSetOutputPortDataType(S, 0, SS_DOUBLE);

    ssSetOutputPortWidth(S, 1, 1);
    ssSetOutputPortDataType(S, 1, SS_DOUBLE);

    ssSetOutputPortWidth(S, 2, 3*MAXBALLS);
    ssSetOutputPortDataType(S, 2, SS_DOUBLE);

    ssSetOutputPortWidth(S, 3, MAXBALLS);
    ssSetOutputPortDataType(S, 3, SS_DOUBLE);

    ssSetOutputPortWidth(S, 4, 1);
    ssSetOutputPortDataType(S, 4, SS_INT32);
}

static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}

static void mdlOutputs(SimStruct *S, int_T tid)
{
    /*inputs*/
    const int_T maxblobs = ssGetInputPortWidth(S, 0);
    const real_T   *centroid  = (const real_T*) ssGetInputPortSignal(S,0);
    const int_T   *bbox  = (const int_T*) ssGetInputPortSignal(S,1);
    const uint16_T   *nblobs  = (const uint16_T*) ssGetInputPortSignal(S,2);
    const uint8_T *Y = (const uint8_T*) ssGetInputPortSignal(S,3);
    const uint8_T *U = (const uint8_T*) ssGetInputPortSignal(S,4);
    const uint8_T *V = (const uint8_T*) ssGetInputPortSignal(S,5);
    const real_T   *pose  = (const real_T*) ssGetInputPortSignal(S,6);
    
    /*outputs*/
    real_T *ball  = (real_T *)ssGetOutputPortRealSignal(S,0);
    real_T *ballconf  = (real_T *)ssGetOutputPortRealSignal(S,1);
    real_T *allballs  = (real_T *)ssGetOutputPortRealSignal(S,2);
    real_T *allballsconf  = (real_T *)ssGetOutputPortRealSignal(S,3);
    int_T *nballs  = (int_T *)ssGetOutputPortRealSignal(S,4);

    int i,ballsize;
    resetoutput(ball,ballconf,allballs,allballsconf,nballs,pose);

    /*printf("nblobs: %d\n",nblobs[0]);*/
    for (i=0;i<nblobs[0];++i)
    {
        /*get ballsize*/
        ballsize = bbox[4*i+2];
        if (bbox[4*i+3]>ballsize) ballsize = bbox[4*i+3];
        
        /*check minimum size*/
        if (ballsize<MINBALLSIZE) continue;
       /* printf("ballsize: %d, c: %f %f\n",ballsize,centroid[2*i],centroid[2*i+1]);*/
        
        /*check ball contrast*/
        if (checkcontrast(ballsize,&centroid[2*i],U,V)) continue;
       /* printf("contrast ok: %f %f\n",centroid[2*i],centroid[2*i+1]);*/
        
        /*get 3D ball position*/
        ball2output(ballsize,&centroid[2*i],ball,ballconf,allballs,allballsconf,nballs,pose);
    }
}

#define MDL_SET_INPUT_PORT_DIMENSION_INFO
static void mdlSetInputPortDimensionInfo(SimStruct        *S,
int_T            port,
const DimsInfo_T *dimsInfo)
{
    if(!ssSetInputPortDimensionInfo(S, port, dimsInfo)) return;
}

# define MDL_SET_OUTPUT_PORT_DIMENSION_INFO
static void mdlSetOutputPortDimensionInfo(SimStruct        *S,
int_T            port,
const DimsInfo_T *dimsInfo)
{
    if(!ssSetOutputPortDimensionInfo(S, port, dimsInfo)) return;
}

static void mdlTerminate(SimStruct *S)
{
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif

