#define S_FUNCTION_NAME findlinepoints_and_obstacles
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"
#include "math.h"
#include "../../Global_par/constants.h"

#define MAXRADIUS 200
#define MAXANGLES 500
#define STEPREDUCTION 1
#define HISTBINS (256>>STEPREDUCTION)
#define NEARBYGREEN 30
#define MAXOBSTACLEDISTANCE 10
#define OBS_MINDARKNESS 50
#define OBS_BLACKCOUNTFACTOR 1700
#define OBS_BLACKSIZE 0.6 /*fraction black of obstacle*/
#define OBS_LIGHTFACTOR 0.06
#define OBS_NSEARCHOBSTACLES 10
#define OBS_MAXOBSTACLEWIDTH 50
#define dabs(A) ((A)>=0?(A):-(A))
#define OBS_RADIUS 0.25
#define OBS_MAXDIFF 0.5 /*in meters*/
#define REMOVEPOINTSBESIDESOBSTACLES 6 /*increase this value if ghost obstacles are visible behind other obstacles!!!*/


static double centerx,centery;
static double par_a,par_b;
static bool initprocedure, lostgreen;
static int xpicpos[MAXANGLES][MAXRADIUS];
static int ypicpos[MAXANGLES][MAXRADIUS];
static int zpicpos[MAXANGLES][MAXRADIUS];
static int bpicpos[MAXANGLES];
static int epicpos[MAXANGLES];
static int maxradius;
static double cosa[MAXANGLES],sina[MAXANGLES];
static double mc[9];
static int x,y,z;
static uint8_T mask[NPIXELS];
static real_T correctionx,correctiony,correctionphi;

/*linepoints detection*/
#define NCIRCLE 32 /*amount of circle points*/
#define YDIST 70 /*allowed green color deviation Y*/
#define UVDIST 13 /*allowed green color deviation UV*/
#define RCLOSE 20 /*radius of check circle near robot in pixels*/
#define RRANGE 210 /*linear decrease range in pixels*/
#define RMIN 6 /*minimum radius of check circle*/
#define MINCONTRAST 40 /*minimum contrast between and side points*/
#define MINLIGHTNESS 40
#define MAXTESTPOINTS 10000 /*maximum amount of testpoints*/
#define GRIDRESOLUTION 0.5 /*grid resolution of testpoints*/
static int ntestpoints;
static double Xtp[MAXTESTPOINTS],Ytp[MAXTESTPOINTS],Xc[NCIRCLE],Yc[NCIRCLE];
#define MULTPOINTDIST 3

static void finetunegreencolor(int* greencolor, const uint8_T** pYUV)
{
    int i,j,m;
    int maincolor[3],distsquare;
    int nGreenPoints = 0;
    
    /*initialize*/
    for (m=0;m<3;++m) maincolor[m] = greencolor[m];
    for (m=0;m<3;++m) greencolor[m] = 0;
    
    /*collect green*/
    for(i=0;i<NANGLES;++i)
    {
        if (bpicpos[i]+NEARBYGREEN>MAXRADIUS) bpicpos[i] = MAXRADIUS-NEARBYGREEN;
        for(j=bpicpos[i];j<bpicpos[i]+NEARBYGREEN;++j);
        {
            for (m=0,distsquare=0;m<3;++m) distsquare += (maincolor[m]-pYUV[m][zpicpos[i][j]])*(maincolor[m]-pYUV[m][zpicpos[i][j]]);
            if (distsquare < HISTBINS*HISTBINS)
            {
                for (m=0;m<3;++m) greencolor[m]+=pYUV[m][zpicpos[i][j]];
                ++nGreenPoints;
            }
        }
    }
    
    /*determine finetuned green*/
    if (nGreenPoints>1) for (m=0;m<3;++m) greencolor[m]/=nGreenPoints;
}

static void detgreencolor(int* greencolor, const uint8_T** pYUV)
{
    int i,j,m;
    int shiftedcolor;
    int colorhist[HISTBINS][3];
    int indmax[3];
    
    /*set colorhist variables to zero*/
    for (m=0;m<3;++m) indmax[m] = 0;
    for (i=0;i<HISTBINS;++i) for (m=0;m<3;++m) colorhist[i][m]=0;
    
    /*collect colors*/
    for(i=0;i<NANGLES;++i)
    {
        if (bpicpos[i]+NEARBYGREEN>MAXRADIUS) bpicpos[i] = MAXRADIUS-NEARBYGREEN;
        for(j=bpicpos[i];j<bpicpos[i]+NEARBYGREEN;++j);
        {
            for (m=0;m<3;++m)
            {
                shiftedcolor = pYUV[m][zpicpos[i][j]] >> STEPREDUCTION;
                if (++colorhist[shiftedcolor][m]>colorhist[indmax[m]][m]) indmax[m]=shiftedcolor;
            }
        }
    }
    
    /*determine coarse main color nearby*/
    for (m=0;m<3;++m) greencolor[m] = (indmax[m] << STEPREDUCTION) + (1 << STEPREDUCTION-1);
    
    /*fine tune main color (assumed to be green)*/
    finetunegreencolor(greencolor, pYUV);
}

static bool isclosetogreen(double* greencolor, int* color,int* L,int* H)
{
    int m;
    bool closetogreen;
    
    /*determine if color is close to green*/
    for (m=0,closetogreen=1;m<3;++m) closetogreen = closetogreen && color[m]>greencolor[m]-L[m] && color[m]<greencolor[m]+H[m];
    return closetogreen;
}

static void detfieldbounds(int* greencolor,const uint8_T** pYUV)
{
    int i,j,m,color[3],low[3],high[3],cnt;
    double green[3];
    low[0]=20;
    low[1]=8;
    low[2]=8;
    high[0]=20;
    high[1]=8;
    high[2]=8;
    
    for (m=0;m<3;++m) green[m] = greencolor[m];
    
    
    for(i=0;i<NANGLES;++i)
    {
        cnt=0;
        epicpos[i]=bpicpos[i];
        for(j=maxradius;j>=bpicpos[i];--j)
        {
            /*get color*/
            for (m=0;m<3;++m) color[m] = pYUV[m][zpicpos[i][j]];
            
            if (isclosetogreen(green,color,low,high))
            {
                ++cnt;
                if (cnt>3)
                {
                   /* epicpos[i]=fmin(maxradius,j+6);*/
                    epicpos[i]=j+6;
                    if (epicpos[i]>maxradius)
                    {
                        epicpos[i]=maxradius;
                    }
                    break;
                }
            }
            /*else cnt=0;*/
        }
    }
}

static int initialcalculations()
{
    int i,j,k;
    double r;
    int nphi;
    
    /*get maximum radius*/
    maxradius = centery-2;
    if (SIZEY-centery<maxradius) maxradius = SIZEY-centery;
    if (MAXRADIUS<maxradius) maxradius = MAXRADIUS;
    
    /*compute linepoints to check*/
    for(i=0;i<NANGLES;++i)
    {
        /*temp. variables*/
        cosa[i] = cos(2.0*PI*i/NANGLES);
        sina[i] = sin(2.0*PI*i/NANGLES);
        
        /*compute pixel locations*/
        for(j=0;j<maxradius;++j)
        {
            x = (int)(centerx-j*cosa[i]);
            y = (int)(centery-j*sina[i]);
            z = x+SIZEX*y;
            xpicpos[i][j]=x;
            ypicpos[i][j]=y;
            zpicpos[i][j]=z;
        }
        
        /*find begin position*/
        k=0;
        bpicpos[i]=-1;
        for (j=0;j<maxradius;++j)
        {
            if (mask[zpicpos[i][j]]==255)
            {
                if (k==0) bpicpos[i]=j;
                ++k;
            }
            else if (k>0) break; /*stop at first bound*/
        }
        
        /*save end pixel location for each angle*/
        epicpos[i]=j-1;
        if (epicpos[i]>maxradius) epicpos[i] = maxradius;
        if (bpicpos[i]==-1) bpicpos[i]=j;
    }
    
    ntestpoints = 0;
    for (r=0.5;r<8;r+=GRIDRESOLUTION)
    {
        nphi = (int)(2.0*PI*r/GRIDRESOLUTION+1);
        for (i=0;i<nphi;++i)
        {
            Xtp[ntestpoints] = centerx+atan(r/par_a)/par_b*cos(i*2.0*PI/nphi);
            Ytp[ntestpoints] = centery+atan(r/par_a)/par_b*sin(i*2.0*PI/nphi);
            ++ntestpoints;
            if (ntestpoints>=MAXTESTPOINTS) break;
        }
        if (ntestpoints>=MAXTESTPOINTS) break;
    }
    
    for (i=0;i<NCIRCLE;++i)
    {
        Xc[i] = cos(i*2*PI/NCIRCLE);
        Yc[i] = sin(i*2*PI/NCIRCLE);
    }
}

/*new get obstacles algorithm*/
static int getobstacles(const uint8_T* Y,real_T* obstacles, uint8_T* obstaclepoints)
{
    int  pixdist,obsind,x,y,z,n,nobstacles,i,j,k,Ydarkest, Ylightest,Yfirst = 0,cnt,obspoints[NANGLES],closestobs,ruler[OBS_MAXOBSTACLEWIDTH],minruler,rulerupper,rulerlower,flag = 0;
    double r,obsphi,obsdist,obswidth,obsx,obsy,maxdist,philower = 0.0,phiupper = 0.0,width;
    
    /*reset obstacles*/
    for (i=0;i<3*NAVOIDABLES;++i) obstacles[i] = 0;
    nobstacles = 0;
    
    /*get darkest and lightest Y*/
    Ydarkest = 255;
    Ylightest = 0;
    for (i=0;i<NANGLES;++i)
    {
        for (j=bpicpos[i];j<maxradius;++j)
        {
            if (Y[zpicpos[i][j]]!=0)
            {
                if (Y[zpicpos[i][j]]<Ydarkest) Ydarkest = Y[zpicpos[i][j]];
                if (Y[zpicpos[i][j]]>Ylightest) Ylightest = Y[zpicpos[i][j]];
            }
        }
    }

    /*stop if no dark objects are found*/
    if (Ydarkest>OBS_MINDARKNESS) return 0;
   
    /*check obstacles on rulers*/
    for (i=0;i<NANGLES;++i)
    {
        cnt = 0;
        for (j=bpicpos[i];j<maxradius;++j)
        {
            if (Y[zpicpos[i][j]]!=0 && Y[zpicpos[i][j]]<Ydarkest+Ylightest*OBS_LIGHTFACTOR)
            {
                obstaclepoints[zpicpos[i][j]] = 1;
                if (cnt==0) Yfirst = j;
                ++cnt;
                if (Yfirst!=0 && cnt>OBS_BLACKCOUNTFACTOR/Yfirst) /*found enough black points*/
                {
                    if (j-Yfirst<OBS_BLACKSIZE) obspoints[i] = Yfirst;/*enough black points close together*/
                    obstaclepoints[zpicpos[i][Yfirst]] = 2;
                    break; /*goto the next ruler*/
                }
            }
            else if (cnt>0) cnt--; /*to be robust for small black spickles*/
            if (Yfirst!=0 && cnt>0 && j-Yfirst>OBS_BLACKCOUNTFACTOR/Yfirst) /*found enough black points*/
            {
                if (OBS_BLACKSIZE*(j-Yfirst)<cnt) obspoints[i] = Yfirst;/*enough black points close together*/
                obstaclepoints[zpicpos[i][Yfirst]] = 2;
                break; /*goto the next ruler*/
            }
        }
        if (cnt>0) /*set not finished object, probably far away*/
        {
            obstaclepoints[zpicpos[i][Yfirst]] = 2;
            obspoints[i] = Yfirst;
        }
        else /*no object found*/
        {
            obstaclepoints[zpicpos[i][maxradius-1]] = 2;
            obspoints[i] = 255;
        }
    }

    /*collect obstacles*/
    nobstacles = 0;
    for (j=0;j<OBS_NSEARCHOBSTACLES;++j)
    {
        /*find closest obstacle*/
        for (i=0,closestobs=0;i<NANGLES;++i) if (obspoints[i]<obspoints[closestobs]) closestobs = i;
        pixdist = obspoints[closestobs];
        if (pixdist>=maxradius) break;
        
        /*find lower and upper bound*/
        maxdist = atan(OBS_MAXDIFF/par_a+tan(par_b*pixdist))/par_b;
        if (maxdist>maxradius) maxdist = maxradius;
        for (rulerupper=closestobs;rulerupper<closestobs+NANGLES;++rulerupper) if (obspoints[rulerupper%NANGLES]>=maxdist) {if (flag==1) break; else flag++;} else {flag=0;}
        for (rulerlower=closestobs;rulerlower>closestobs-NANGLES;--rulerlower) if (obspoints[rulerlower%NANGLES]>=maxdist) {if (flag==1) break; else flag++;} else {flag=0;}
        
        /*compute obstacle parameters*/
        obsphi = (rulerupper+rulerlower)*PI/NANGLES;
        obsdist = pixdist;
        obswidth = rulerupper-rulerlower-2;
       
       /*add obstacles to output*/
        if (atan(MAXOBSTACLEDISTANCE/par_a)>par_b*obsdist)
        {
            /*compute distance of obstacle*/
            r = par_a*tan(par_b*obsdist); /*get meter distance*/
            
            /*check size of obstacle(s)*/
  /*          width = obswidth*2*PI/NANGLES*r;
            if (width>4*OBS_RADIUS && obswidth>2)
            {
                 n = width/2.0/OBS_RADIUS;
                 philower = rulerlower*2*PI/NANGLES+1.*OBS_RADIUS/r;
                 phiupper = rulerupper*2*PI/NANGLES-1.*OBS_RADIUS/r;
            }
            else */
                n = 1;
            
            /*add obstacle*/
            for (i=0;i<n;++i)
            {
                if (nobstacles<NAVOIDABLES)
                {
                    if (n>1) obsphi = philower+(phiupper-philower)*i/(n-1.0);
                               
                    /*log info*/
                    x = (int)(centerx-obsdist*cos(obsphi));
                    y = (int)(centery-obsdist*sin(obsphi));
                    z = x+SIZEX*y;
                    obstaclepoints[z]=5;
                    
                    /*add obstacle*/
                    obstacles[3*nobstacles+2] = OBS_RADIUS; /*size of obstacle is made depended on the distance to the robot*/
                    obsx = -(r+obstacles[3*nobstacles+2])*sin(obsphi) - correctionx;
                    obsy = (r+obstacles[3*nobstacles+2])*cos(obsphi) - correctiony;
                    obstacles[3*nobstacles] = cos(correctionphi)*obsx+sin(correctionphi)*obsy;
                    obstacles[3*nobstacles+1] = -sin(correctionphi)*obsx+cos(correctionphi)*obsy;
                    ++nobstacles;
                }
            }
        } /*else obstacle is too far away*/

        for (k=rulerlower-REMOVEPOINTSBESIDESOBSTACLES;k<rulerupper+REMOVEPOINTSBESIDESOBSTACLES;++k) obspoints[k%NANGLES] = 255;
    }
    return nobstacles;
}

static void getlinepoints(const uint8_T** pYUV,int* greencolor,real_T* linepoints,int_T* nlinepoints,real_T* linepoints_image)
{
    int d,i,j,k,low[3],high[3],flag;
    int sel,color[3],color2[3];
    int Ymax,z[NCIRCLE];
    double x,y;
    double green[3];
    const uint8_T* Y;
    real_T r,a;
    double tmp1,tmp2;
    
    /*get Y channel*/
    Y = pYUV[0];
    
    /*determine lower and upper threshold for green check*/
    low[0]=YDIST;
    low[1]=UVDIST;
    low[2]=UVDIST;
    high[0]=255;
    high[1]=UVDIST;
    high[2]=UVDIST;
    
    /*select points*/
    nlinepoints[0] = 0;
    for (k=0;k<ntestpoints;++k)
    {
        /*get point*/
        x = Xtp[k];
        y = Ytp[k];
        
        /*choose filter size*/
        tmp1 = sqrt((centerx-x)*(centerx-x)+(centery-y)*(centery-y));
        d = (int)(RCLOSE*(1-tmp1/RRANGE)+.5);
        if (d<RMIN) d = RMIN;
        
        /*find max*/
        for (j=0;j<2;++j)
        {
            if (x<=2*d+1 || x>=640-2*d-2 || y<=2*d+1 || y>=480-2*d-2) continue;
            sel = x+2*d;
            for (i=x-2*d;i<x+2*d;++i) if (Y[i+640*(int)y]>Y[sel+640*(int)y]) sel = i;
            x = sel;
            sel = y+2*d;
            for (i=y-2*d;i<y+2*d;++i) if (Y[(int)x+640*i]>Y[(int)x+640*sel]) sel = i;
            y = sel;
        }
        
        /*check border*/
        if (x<=d+1 || x>=640-d-2 || y<=d+1 || y>=480-d-2) continue;
        Ymax = Y[(int)(x+.5)+640*((int)(y+.5))];
        
        /*finetune x center*/
        tmp1=0;
        tmp2=0;
        for (i=x;i>x-d;--i)
        {
            if ((int)(Y[i+640*(int)(y+.5)]) <= Ymax-MINCONTRAST) break;
            tmp1 += (Y[i+640*(int)(y+.5)]-(Ymax-MINCONTRAST));
            tmp2 += (Y[i+640*(int)(y+.5)]-(Ymax-MINCONTRAST))*i;
        }
        for (j=x;j<x+d;++j)
        {
            if ((int)(Y[j+640*(int)(y+.5)]) <= Ymax-MINCONTRAST) break;
            tmp1 += (Y[j+640*(int)(y+.5)]-(Ymax-MINCONTRAST));
            tmp2 += (Y[j+640*(int)(y+.5)]-(Ymax-MINCONTRAST))*j;
        }
        x = tmp2/tmp1;
        
        /*finetune y center*/
        tmp1=0;
        tmp2=0;
        for (i=y;i>y-d;--i)
        {
            if ((int)(Y[(int)(x+.5)+640*i]) <= Ymax-MINCONTRAST) break;
            tmp1 += (Y[(int)(x+.5)+640*i]-(Ymax-MINCONTRAST));
            tmp2 += (Y[(int)(x+.5)+640*i]-(Ymax-MINCONTRAST))*i;
        }
        for (j=y;j<y+d;++j)
        {
            if ((int)(Y[(int)(x+.5)+640*j]) <= Ymax-MINCONTRAST) break;
            tmp1 += (Y[(int)(x+.5)+640*j]-(Ymax-MINCONTRAST));
            tmp2 += (Y[(int)(x+.5)+640*j]-(Ymax-MINCONTRAST))*j;
        }
        y = tmp2/tmp1;
        
        /*get intensity of center*/
        Ymax = Y[(int)(x+.5)+640*((int)(y+.5))];
        
        /*circle check*/
        sel = 0;
        z[sel] = (int)(d*Xc[sel]+x+.5)+((int)(d*Yc[sel]+y+.5))*640;
        for (i=1;i<NCIRCLE;++i) /*find maximum at circle*/
        {
            z[i] = (int)(d*Xc[i]+x+.5)+((int)(d*Yc[i]+y+.5))*640;
            if (Y[z[i]]>Y[z[sel]]) sel = i;
        }
        
        /*get green*/
        for (j=0;j<3;++j) green[j] = greencolor[j];
        if (Ymax<green[0]+MINLIGHTNESS) continue;
        
        /*check one side of line*/
        i = (sel+(NCIRCLE>>2))%NCIRCLE;
        for (j=0;j<3;++j) color[j] = pYUV[j][z[i]];
        if (!isclosetogreen(green,color,low,high)) continue;
        if (Y[z[sel]]<color[0]+MINCONTRAST || Ymax<color[0]+MINCONTRAST || Y[z[i]]==0) continue;
        
        /*check other side of line*/
        i = (sel+(NCIRCLE>>2)+(NCIRCLE>>1))%NCIRCLE;
        for (j=0;j<3;++j) color2[j] = pYUV[j][z[i]];
        if (!isclosetogreen(green,color2,low,high)) continue;
        if (Y[z[sel]]<color2[0]+MINCONTRAST || Ymax<color2[0]+MINCONTRAST || Y[z[i]]==0) continue;
        
        if (fabs(color[0]-color2[0])>100 || fabs(color[1]-color2[1])>4 || fabs(color[2]-color2[2])>4) continue;
        
        /*add linepoint*/
        if (nlinepoints[0]<MAXLINEPOINTS)
        {
            flag = 0;
            for (j=0;j<nlinepoints[0];++j)
            {
                if (dabs(x-linepoints_image[j])<MULTPOINTDIST && dabs(y-linepoints_image[j+MAXLINEPOINTS])<MULTPOINTDIST) flag = 1;
            }
            if (flag) continue;
            r = sqrt((centerx-x)*(centerx-x)+(centery-y)*(centery-y));
            a = atan2(centery-y,centerx-x);
            if (atan(10/par_a)>par_b*r) r = par_a*tan(par_b*r); /*get meter distance*/
            else r = 10; /*point is too far away*/
            
            linepoints[nlinepoints[0]] = -r*sin(a);
            linepoints[nlinepoints[0]+MAXLINEPOINTS] = r*cos(a);
            linepoints_image[nlinepoints[0]] = x;
            linepoints_image[nlinepoints[0]+MAXLINEPOINTS] = y;
            ++nlinepoints[0];
        }
    }
}

static void local2global(real_T* obstacles_local,real_T* obstacles,int_T* nobstacles,const real_T* pose)
{
    int i;
    for(i = 0; i < nobstacles[0]*3; i+=3)
    {
        obstacles[i] = cos(pose[2])*obstacles_local[i] - sin(pose[2])*obstacles_local[i+1] + pose[0];
        obstacles[i+1] = sin(pose[2])*obstacles_local[i] + cos(pose[2])*obstacles_local[i+1] + pose[1];
        obstacles[i+2] = obstacles_local[i+2]; /*Radius*/
    }
    for(i = nobstacles[0]*3; i < NAVOIDABLES*3; i+=3)
    {
        obstacles[i] = 10; /*Self[0]; /*Opp[cnt]; /*cos(phi)*dx + sin(phi)*dy;*/
        obstacles[i+1] = 10; /*Self[1]; /*Opp[cnt+1]; /*-sin(phi)*dx + cos(phi)*dy;*/
        obstacles[i+2] = 0.0; /*Radius*/
    }
}


/*********************************************************************/
static void mdlInitializeSizes(SimStruct *S)
{
    DECL_AND_INIT_DIMSINFO(inputDimsInfo);
    DECL_AND_INIT_DIMSINFO(outputDimsInfo);
    
    FILE *fid;
    
    /*PARAMETERS*/
    ssSetNumSFcnParams(S, 0);
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }
    initprocedure=1;
    
    fid=fopen("mc.dat","rb");
    if (fid!=NULL)
    {
        fread(mc,sizeof(double),9,fid);
        fclose(fid);
        
        centerx = mc[0];
        centery = mc[1];
        par_a = mc[2];
        par_b = mc[3];
        correctionx = mc[4];
        correctiony = mc[5];
        correctionphi = mc[8];
    }
    else
    {
        printf("could not find mc.dat!!!\n");
        return;
    }
    
    fid=fopen("mask.dat","rb");
    if (fid!=NULL)
    {
        fread(mask,1,NPIXELS,fid);
        fclose(fid);
    }
    else
    {
        printf("mask.dat is not available!!!\n");
        return;
    }
    
    /*INPUT PORTS*/
    if (!ssSetNumInputPorts(S, 4)) return;
    
    inputDimsInfo.width = SIZEX;
    ssSetInputPortDimensionInfo(S, 0, &inputDimsInfo);
    ssSetInputPortMatrixDimensions(  S ,0, SIZEX, SIZEY);
    ssSetInputPortDataType(S, 0, SS_UINT8);
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortRequiredContiguous(S, 0, 1);
    
    inputDimsInfo.width = SIZEX;
    ssSetInputPortDimensionInfo(S, 1, &inputDimsInfo);
    ssSetInputPortMatrixDimensions(  S ,1, SIZEX, SIZEY);
    ssSetInputPortDataType(S, 1, SS_UINT8);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
    ssSetInputPortRequiredContiguous(S, 1, 1);
    
    inputDimsInfo.width = SIZEX;
    ssSetInputPortDimensionInfo(S, 2, &inputDimsInfo);
    ssSetInputPortMatrixDimensions(  S ,2, SIZEX, SIZEY);
    ssSetInputPortDataType(S, 2, SS_UINT8);
    ssSetInputPortDirectFeedThrough(S, 2, 1);
    ssSetInputPortRequiredContiguous(S, 2, 1);
     
    ssSetInputPortWidth(S, 3, 3);
    ssSetInputPortDataType(S, 3, SS_DOUBLE);
    ssSetInputPortDirectFeedThrough(S, 3, 1);
    ssSetInputPortRequiredContiguous(S, 3, 1);
    
    /*OUTPUT PORTS*/
    if (!ssSetNumOutputPorts(S, 7)) return;
    
    ssSetOutputPortWidth(S, 0, 3*NAVOIDABLES);
    ssSetOutputPortDataType(S, 0, SS_DOUBLE);
    
    ssSetOutputPortWidth(S, 1, 1);
    ssSetOutputPortDataType(S, 1, SS_INT32);
    
    outputDimsInfo.width = MAXLINEPOINTS;
    ssSetOutputPortDimensionInfo(S, 2, &outputDimsInfo);
    ssSetOutputPortMatrixDimensions( S ,2, MAXLINEPOINTS, 2);
    ssSetOutputPortDataType(S, 2, SS_DOUBLE);
    
    ssSetOutputPortWidth(S, 3, 1);
    ssSetOutputPortDataType(S, 3, SS_INT32);
    
    outputDimsInfo.width = MAXLINEPOINTS;
    ssSetOutputPortDimensionInfo(S, 4, &outputDimsInfo);
    ssSetOutputPortMatrixDimensions( S ,4, MAXLINEPOINTS, 2);
    ssSetOutputPortDataType(S, 4, SS_DOUBLE);
    
    ssSetOutputPortWidth(S, 5, 3);
    ssSetOutputPortDataType(S, 5, SS_INT32);
    
        outputDimsInfo.width = SIZEX;
    ssSetOutputPortDimensionInfo(S, 6, &outputDimsInfo);
    ssSetOutputPortMatrixDimensions( S ,6, SIZEX, SIZEY);
    ssSetOutputPortDataType(S, 6, SS_UINT8);

}

static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}

static void mdlOutputs(SimStruct *S, int_T tid)
{
    /*inputs*/
    const uint8_T *Y  = (const uint8_T*) ssGetInputPortSignal(S,0);
    const uint8_T *U  = (const uint8_T*) ssGetInputPortSignal(S,1);
    const uint8_T *V  = (const uint8_T*) ssGetInputPortSignal(S,2);
    const real_T *pose  = (const real_T*) ssGetInputPortSignal(S,3);
    
    /*outputs*/
    real_T *obstacles  = (real_T *)ssGetOutputPortRealSignal(S,0);
    int_T *nobstacles  = (int_T *)ssGetOutputPortRealSignal(S,1);
    real_T *linepoints  = (real_T *)ssGetOutputPortRealSignal(S,2);
    int_T *nlinepoints  = (int_T *)ssGetOutputPortRealSignal(S,3);
    real_T *linepoints_image = (real_T *)ssGetOutputPortRealSignal(S,4);
    int_T *greencolor = (int_T *)ssGetOutputPortRealSignal(S,5);
    uint8_T *obstaclepoints = (uint8_T *)ssGetOutputPortRealSignal(S,6);
    
    int i;
    real_T obstacles_local[3*NAVOIDABLES];

    /*get pointer to Y U and V*/
    const uint8_T** pYUV;
    pYUV = (const uint8_T**)malloc(3*sizeof(const uint8_T*));
    pYUV[0] = Y;
    pYUV[1] = U;
    pYUV[2] = V;
    
    for (i=0;i<NPIXELS;++i) obstaclepoints[i] = 0;
    
    if (initprocedure) /*-------------------------------------------------------------*/
    {
        /*do initialization once*/
        initprocedure=0;
        
        /*do initial calculations of pixel locations in each line search direction*/
        initialcalculations();
    }
    else
    {
        /*determine green color*/
        detgreencolor(greencolor,pYUV);
        
        /*get linepoints*/
        getlinepoints(pYUV,greencolor,linepoints,nlinepoints,linepoints_image);
        
        /*get obstacles*/
        detfieldbounds(greencolor,pYUV);
        nobstacles[0] = getobstacles(Y,obstacles_local,obstaclepoints);
        local2global(obstacles_local,obstacles,nobstacles,pose);
   }
    free(pYUV);
}

static void mdlTerminate(SimStruct *S)
{
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
