#define S_FUNCTION_NAME findpose
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"
#include <math.h>
#include "../../Global_par/constants.h"

#define NOPTIMIZE 20

static int prevposfound;
static double prevbestpose[NP];
static const real_T *pwhitepoints;
static int nwhitepoints;
static uint8_T fieldmap[FIELDMAP_Y][FIELDMAP_X];
static int_T maxiterations; /*maximum amount of iterations of the Levenberg - Marquardt algorithm*/
static real_T allowedmeanerror; /*maximum mean distance of a white point to the nearest line*/
static real_T correctionx, correctiony,correctionphi;
static real_T mc[9];
static real_T J,Jtmp,posetmp[3];
static real_T flipradius, prevencrobot[3];
#define POSVAR .1
#define COSTFCNINCREASE 1.0

/********************************************/
static void getrandpose(double *pose)
{
    pose[0]=FIELDWIDTH*((double)rand()/RAND_MAX-0.5);
    pose[1]=FIELDLENGTH*((double)rand()/RAND_MAX-0.5);
    pose[2]=2*PI*rand()/RAND_MAX;
}

/*****************simplex optimization method***********************/
#define SWAP(a,b) {swap=(a);(a)=(b);(b)=swap;}
static void simplex(double *p, double *y, int ndim, double ftol,
double (*funk)(double *), int *nfunk);
static double simplextry(double *p, double y[], double psum[], int ndim,
double (*funk)(double *), int ihi, double fac);
static double costfcn(double *p);
/*******************end simplex optimization method*****************/

static double optimize(double *pose)
{
    int i,j,nfunc;
    double x[NP],y[MP],p[MP*NP];
    
    /*generate initial simplex*/
    for (i=0;i<MP;i++)
    {
        for (j=0;j<NP;j++) x[j]=p[i+MP*j]=(i == (j+1) ? INITSIMPLEXSIZE : 0.0)+pose[j];
        y[i]=costfcn(x); /*compute cost-function for all vertices*/
    }
    
        /*do simplex optimization*/
    simplex(p,y,NP,FTOL,costfcn,&nfunc);
    
        /*assign first vertex to pose*/
    for (i=0;i<NP;++i) pose[i]=p[i*MP];
    
            /*check if threshold requirement is satisfied*/
        /*printf("%f < %f\n",costfcn(pose),allowedmeanerror);*/
    return costfcn(pose);
}

static void mdlInitializeSizes(SimStruct *S)
{
    DECL_AND_INIT_DIMSINFO(inputDimsInfo);
    const real_T *pr;
    FILE *fid;
    int i;
    
    /*PARAMETERS*/
    ssSetNumSFcnParams(S, 1);
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }
    
    pr = (const real_T *)mxGetPr(ssGetSFcnParam(S,0));
    maxiterations = pr[0];
    allowedmeanerror = pr[1];
    
    fid=fopen("mc.dat","rb");
    if (fid!=NULL)
    {
        fread(mc,sizeof(double),9,fid);
        fclose(fid);
        
        correctionx = mc[4];
        correctiony = mc[5];
        correctionphi = mc[8];
    }
    else
    {
        printf("could not find mc.dat!!!");
        return;
    }
    
    fid=fopen("fieldmap.dat","rb");
    if (fid!=NULL)
    {
        fread(fieldmap,1,FIELDMAP_X*FIELDMAP_Y,fid);
        fclose(fid);
    }
    else
    {
        printf("could not find fieldmap.dat!!!");
        return;
    }
    
    for (i=0;i<3;++i)  prevbestpose[i]=0;
    prevposfound = 0;
    flipradius = 1e10;
    
    /*INPUT PORTS*/
    if (!ssSetNumInputPorts(S, 3)) return;
    
    inputDimsInfo.width = MAXLINEPOINTS;
    ssSetInputPortDimensionInfo(S, 0, &inputDimsInfo);
    ssSetInputPortMatrixDimensions(  S ,0, MAXLINEPOINTS, 2);
    ssSetInputPortDataType(S, 0, SS_DOUBLE);
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortRequiredContiguous(S, 0, 1);
    
    ssSetInputPortWidth(S, 1, 1);
    ssSetInputPortDataType(S, 1, SS_INT32);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
    ssSetInputPortRequiredContiguous(S, 1, 1);
    
    ssSetInputPortWidth(S, 2, 3);
    ssSetInputPortDataType(S, 2, SS_DOUBLE);
    ssSetInputPortDirectFeedThrough(S, 2, 1);
    ssSetInputPortRequiredContiguous(S, 2, 1);
    
    /*OUTPUT PORTS*/
    if (!ssSetNumOutputPorts(S, 3)) return;
    
    ssSetOutputPortWidth(S, 0, 3);
    ssSetOutputPortDataType(S, 0, SS_DOUBLE);
    
    ssSetOutputPortWidth(S, 1, 1);
    ssSetOutputPortDataType(S, 1, SS_INT32);
    
    ssSetOutputPortWidth(S, 2, 1);
    ssSetOutputPortDataType(S, 2, SS_DOUBLE);
}

static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, CONTINUOUS_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}

#define MDL_START  /* Change to #undef to remove function */
#if defined(MDL_START)
static void mdlStart(SimStruct *S)
{
}
#endif /*  MDL_START */

static void mdlOutputs(SimStruct *S, int_T tid)
{
    /*inputs*/
    const real_T *whitepoints  = (const real_T*) ssGetInputPortSignal(S,0);
    const int_T  *nwp  = (const int_T*) ssGetInputPortSignal(S,1);
    const real_T  *encrobot  = (const real_T*) ssGetInputPortSignal(S,2);
    
    /*outputs*/
    real_T        *pose  = (real_T *)ssGetOutputPortRealSignal(S,0);
    int_T        *posfound  = (int_T *)ssGetOutputPortRealSignal(S,1);
    real_T        *fitval  = (real_T *)ssGetOutputPortRealSignal(S,2);
    
    /*variables*/
    int i,j,best;
    double J,Js[NOPTIMIZE],poses[NOPTIMIZE][NP];
    double testpose[3];
    
    /*check amount of whitepoints*/
    nwhitepoints=nwp[0];
    pwhitepoints = whitepoints;

   /* printf("nlinepoints %d\n",nwhitepoints);*/
    
    if (nwhitepoints>MINLINEPOINTS)
    {
        /*---attempt 1---*/
        if (prevposfound) for (i=0;i<NP;++i) poses[0][i]=prevbestpose[i];
        else getrandpose(&poses[0][0]);
        Js[0] = optimize(&poses[0][0]);
        best = 0;
        
        /*---other attempts---*/
        for (i=1;i<NOPTIMIZE;++i)
        {
            getrandpose(&poses[i][0]);
            Js[i] = optimize(&poses[i][0]);
            if (Js[i]<Js[best]) best = i;
        }
        J = Js[best];
        for (i=0;i<NP;++i) pose[i]=poses[best][i];
        
        fitval[0] = J;
        
        if (((J<allowedmeanerror) || (J<allowedmeanerror && fabs(pose[0])<(FIELDWIDTH*.5-.5))) &&
        (fabs(prevbestpose[0])-fabs(poses[best][0]))*(fabs(prevbestpose[0])-fabs(poses[best][0]))+(fabs(prevbestpose[1])-fabs(poses[best][1]))*(fabs(prevbestpose[1])-fabs(poses[best][1]))<flipradius*flipradius)
        {
            for (i=0;i<NP;++i) prevbestpose[i]=poses[best][i];
            posfound[0]=1;
            
            /*check locality of solution*/
            posetmp[0]=pose[0]+POSVAR;
            posetmp[1]=pose[1];
            posetmp[2]=pose[2];
            Jtmp = costfcn(posetmp);/*printf("posvar J: %f\n",Jtmp);*/
            if (Jtmp<J*COSTFCNINCREASE) posfound[0]=0;
            posetmp[0]=pose[0]-POSVAR;
            Jtmp = costfcn(posetmp);/*printf("posvar J: %f\n",Jtmp);*/
            if (Jtmp<J*COSTFCNINCREASE) posfound[0]=0;
            posetmp[0]=pose[0];
            posetmp[1]=pose[1]+POSVAR;
            Jtmp = costfcn(posetmp);/*printf("posvar J: %f\n",Jtmp);*/
            if (Jtmp<J*COSTFCNINCREASE) posfound[0]=0;
            posetmp[1]=pose[1]-POSVAR;
            Jtmp = costfcn(posetmp);/*printf("posvar J: %f\n",Jtmp);*/
            if (Jtmp<J*COSTFCNINCREASE) posfound[0]=0;
            
            if (posfound[0]==1)
            {
                pose[0]+=correctionx*cos(pose[2])-correctiony*sin(pose[2]);
                pose[1]+=correctionx*sin(pose[2])+correctiony*cos(pose[2]);
            }
            else for (i=0;i<NP;++i) pose[i]=0; /*currentpose[i];*/
        }
        else
        {
            for (i=0;i<NP;++i) pose[i]=0; /*currentpose[i];*/
            posfound[0]=0;
        }
    }
    else /*not enough white points*/
    {
        for (i=0;i<NP;++i) pose[i]=0;/*currentpose[i];*/
        posfound[0]=0;
        fitval[0]=1.0;
    }
    
    /*angle correction*/
    pose[2]+=correctionphi;
    pose[2]=fmod(M_PI+pose[2],2*M_PI)-M_PI;
    if (pose[2]<-M_PI) pose[2]+=2*M_PI;
    
    /*add absolute distance moved based on encoders to flipradius if the position is not found*/
    if (posfound[0]==0 && prevposfound==0)
    {
        flipradius+=sqrt((encrobot[0]-prevencrobot[0])*(encrobot[0]-prevencrobot[0])+(encrobot[1]-prevencrobot[1])*(encrobot[1]-prevencrobot[1]));
        flipradius+=1.0/32.0; /*increase flipradius if a position is found 1 m/s*/
    }
    else
    {
        flipradius = .5; /*set allowable distance between new and old pose*/
    }
    
    /*update previous-parameters for next function call*/
    prevposfound=posfound[0]; /*set prevposfound use in the next sample*/
    prevencrobot[0]=encrobot[0];
    prevencrobot[1]=encrobot[1];
    prevencrobot[2]=encrobot[2];
}

/****************************************************************/

static double costfcn(double *p)
{
    int i,Xi,Yi;
    double X,Y,sumx=0,hx=0,cosp2,sinp2;
    const double *x, *y;
    
    /*assign white point variables*/
    x = (const real_T *)pwhitepoints;
    y = (const real_T *)(pwhitepoints+MAXLINEPOINTS);
    
    /*this reduces calculation time by a factor 5*/
    cosp2=cos(p[2]);
    sinp2=sin(p[2]);
    for (i=0;i<nwhitepoints;++i)
    {
        /*local to global transformation*/
        X = p[0]+x[i]*cosp2-y[i]*sinp2;
        Y = p[1]+x[i]*sinp2+y[i]*cosp2;
        
        /*convert to matrix coordinates*/
        Xi=X*FIELDMAP_RESOLUTION+((FIELDMAP_X-1)>>1);
        Yi=Y*FIELDMAP_RESOLUTION+((FIELDMAP_Y-1)>>1);
        
        if (Xi>=FIELDMAP_X || Xi<0 || Yi>=FIELDMAP_Y || Yi<0) sumx+=255;
        else sumx+=fieldmap[Yi][Xi];
    }
    return sumx/nwhitepoints/256;
}

/****************************************************************/

static double simplextry(double *p, double y[], double psum[], int ndim, double (*funk)(double *), int ihi, double fac)
{
    int j;
    double fac1,fac2,ytry,*ptry;
    
    ptry=(double *)malloc(sizeof(double)*ndim);
    fac1=(1.0-fac)/ndim;
    fac2=fac1-fac;
    for (j=0;j<ndim;++j) ptry[j]=psum[j]*fac1-p[ihi+(ndim+1)*j]*fac2;
    ytry=(*funk)(ptry);
    if (ytry < y[ihi])
    {
        y[ihi]=ytry;
        for (j=0;j<ndim;++j)
        {
            psum[j] += ptry[j]-p[ihi+(ndim+1)*j];
            p[ihi+(ndim+1)*j]=ptry[j];
        }
    }
    free(ptry);
    return ytry;
}

/****************************************************************/

static void simplex(double *p, double *y, int ndim, double ftol, double (*funk)(double *), int *nfunk)
{
    int i,ihi,ilo,inhi,j,mpts=ndim+1;
    double rtol,sum,swap,ysave,ytry,*psum;
    
    psum=(double *)malloc(sizeof(double)*ndim);
    *nfunk=0;
    
    for (j=0;j<ndim;j++)
    {
        for (sum=0.0,i=0;i<mpts;i++) sum += p[i+(ndim+1)*j];
        psum[j]=sum;
    }
    
    for (;;)
    {
        ilo=0;
        ihi = y[0]>y[1] ? (inhi=1,0) : (inhi=0,1);
        for (i=0;i<mpts;++i)
        {
            if (y[i] <= y[ilo]) ilo=i;
            if (y[i] > y[ihi])
            {
                inhi=ihi;
                ihi=i;
            }
            else if (y[i] > y[inhi] && i != ihi) inhi=i;
        }
        rtol=2.0*fabs(y[ihi]-y[ilo])/(fabs(y[ihi])+fabs(y[ilo])+TINY);
        if (rtol < ftol)
        {
            SWAP(y[0],y[ilo])
            for (i=0;i<ndim;++i) SWAP(p[0+(ndim+1)*i],p[ilo+(ndim+1)*i])
            break;
        }
        if (*nfunk >= maxiterations)
        {
            free(psum);
            return;
        }
        *nfunk += 2;
        ytry=simplextry(p,y,psum,ndim,funk,ihi,-1.0);
        if (ytry <= y[ilo])
            ytry=simplextry(p,y,psum,ndim,funk,ihi,2.0);
        else if (ytry >= y[inhi])
        {
            ysave=y[ihi];
            ytry=simplextry(p,y,psum,ndim,funk,ihi,0.5);
            if (ytry >= ysave)
            {
                for (i=0;i<mpts;++i)
                {
                    if (i != ilo)
                    {
                        for (j=0;j<ndim;++j) p[i+(ndim+1)*j]=psum[j]=0.5*(p[i+(ndim+1)*j]+p[ilo+(ndim+1)*j]);
                        y[i]=(*funk)(psum);
                    }
                }
                *nfunk += ndim;
                
                for (j=0;j<ndim;j++) 
                {
                    for (sum=0.0,i=0;i<mpts;i++) sum += p[i+(ndim+1)*j];
                    psum[j]=sum;
                }
            }
        }
        else --(*nfunk);
    }
    free(psum);
}

#undef MDL_INITIALIZE_CONDITIONS   /* Change to #undef to remove function */
#undef MDL_UPDATE  /* Change to #undef to remove function */
#undef MDL_DERIVATIVES  /* Change to #undef to remove function */

static void mdlTerminate(SimStruct *S)
{
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
