#ifndef GLOBAL_PAR_LIST
#define GLOBAL_PAR_LIST

/*
    global_par.h

    global parameters for techunited software

    IMPORTANT: parameters should be defined as scalar numbers!

    c-programmers: include this file in your code
    m-programmers: use get_global_par.m to access parameters from this file

    Rene van de Molengraft, November, 10th, 2006

    Last updated: Roel Merry, April 10, 2007, Switch to field sizes for 2007: #define largefield
                  WA/RvdM, April 14, 2007, changed to real refbox values
                  RvdM, March, 20th, 2008, added soccer roles
*/

#define MAXNOBJ_LOCAL               10      /* as tracked by local tracker */
#define MAXNOBJ_GLOBAL              12      /* as tracked by global tracker */

#define NPATH_MAX                   50      /* maximum length of the global path (waypoints) */
#define NOBST_TEAMMEMBERS_MAX       5       /* maximum number of obstacles per path of team members */

#define MAX_TURTLES                 7
#define TRC_NUMBER_OF_TURTLES       6
#define MAX_ACTIVE_TURTLES          6
#define MAX_OPPONENTS               10
#define NAVOIDABLES                 50      /* maximum amount of avoidables that can be found by vision */
#define NAVOIDABLES_FOR_STRATEGY    50      /* also defined in init_motion.m! */

#define MAX_DEVPCS                  20

/* refbox commands (as produced by the actual refbox computer and the refbox simulator) */
#define COMM_STOP                   'S'
#define COMM_START                  's'

/* general game flow commands */
#define COMM_START_FIRST_HALF       '1'
#define COMM_HALF_TIME              'h'
#define COMM_START_SECOND_HALF      '2'
#define COMM_END_GAME               'e'

#define COMM_PARKING                'L'
#define COMM_CANCEL                 'x'

#define COMM_GOAL_MAGENTA           'a'
#define COMM_GOAL_CYAN              'A'
#define COMM_SUBGOAL_MAGENTA        'd'
#define COMM_SUBGOAL_CYAN           'D'
/* refbox commands */

#define COMM_KICKOFF_MAGENTA        'k'
#define COMM_KICKOFF_CYAN           'K'

#define COMM_FREEKICK_MAGENTA       'f'
#define COMM_FREEKICK_CYAN          'F'

#define COMM_GOALKICK_MAGENTA       'g'
#define COMM_GOALKICK_CYAN          'G'

#define COMM_THROWIN_MAGENTA        't'
#define COMM_THROWIN_CYAN           'T'

#define COMM_CORNER_MAGENTA         'c'
#define COMM_CORNER_CYAN            'C'

#define COMM_PENALTY_MAGENTA        'p'
#define COMM_PENALTY_CYAN           'P'

#define COMM_DROPPED_BALL           'N'

#define COMM_REPAIR_OUT_MAGENTA     'o'
#define COMM_REPAIR_OUT_CYAN        'O'
#define COMM_REPAIR_IN_MAGENTA      'i'
#define COMM_REPAIR_IN_CYAN         'I'


/* refbox commands (as handled by refbox handler) */
#define COMM_KICKOFF_ACTIVE         30
#define COMM_FREEKICK_ACTIVE        31
#define COMM_GOALKICK_ACTIVE        32
#define COMM_THROWIN_ACTIVE         33
#define COMM_CORNER_ACTIVE          34
#define COMM_PENALTY_ACTIVE         35

#define COMM_DROPPED_BALL_TE        36
#define COMM_START_TE               37
#define COMM_STOP_TE                38

#define COMM_KICKOFF_PASSIVE        39
#define COMM_FREEKICK_PASSIVE       40
#define COMM_GOALKICK_PASSIVE       41
#define COMM_THROWIN_PASSIVE        42
#define COMM_CORNER_PASSIVE         43
#define COMM_PENALTY_PASSIVE        44

#define COMM_PARKING_TE             45
#define COMM_GO_IN_FIELD_TE         46

#define COMM_START_FIRST_HALF_TE    47
#define COMM_START_SECOND_HALF_TE   48



/* soccer role definition */
#define MAX_GAME_ROLES              7
#define ROLE_NONE                   -1
#define ROLE_ATTACKER_MAIN          1
#define ROLE_ATTACKER_ASSIST        2
#define ROLE_DEFENDER_MAIN          3
#define ROLE_DEFENDER_ASSIST1       4
#define ROLE_DEFENDER_ASSIST2       5
#define ROLE_GOALKEEPER             6
#define ROLE_GOHOME                 7
#define ROLE_SEARCHBALL             8
#define MAX_ROLES                   9   /* total number of roles in 'role_handler' mode */
/* refbox role definitions */
#define MAX_REFBOX_ROLES            6
#define ROLE_REFBOX_NONE            -1
#define ROLE_REFBOX_1               1
#define ROLE_REFBOX_2               2
#define ROLE_REFBOX_3               3
#define ROLE_REFBOX_4               4
#define ROLE_REFBOX_5               5
/* For mu positioning */
#define MAX_PEERS                   6   /* dmax(MAX_GAME_ROLES,MAX_REFBOX_ROLES) */

/* MSL Refbox Rules (MSLRR) 2011 */
#define MSLRR_MIN_DIST_TO_BALL_REFBOX_PASSIVE      3.0
#define MSLRR_MIN_DIST_TO_BALL_REFBOX_ACTIVE       2.0



/* Tunable parameter defines */
#define MAX_TUNABLE_PARS            100
#define MAX_NAME_LEN_TUNABLE_PARS   30
#define REQMSG_GET_NAMES            0
#define REQMSG_GET_DEFAULTS         1
#define REQMSG_SET_PARS             2

/* Tunable parameters index */
/* Worldmodel */
#define WMTuneparams                [2:14]    /* Global world model */
#define WMLiteTuneparams            [15:21]     /* tunable parameters for worldmodel lite */

/* strategy pktdim */
#define STRAT_PKT_DIM               8

/*---field dependent parameters begin---*/
#define DEFAULTFIELDSELECTION 'TUE_field'
#define CORNERRADIUS 0.500000
#define CIRCLERADIUS 1.000000
#define FIELDWIDTH 8.000000
#define FIELDLENGTH 12.000000
#define PENALTYAREAWIDTH 5.000000
#define PENALTYAREALENGTH 1.500000
#define GOALAREAWIDTH 3.000000
#define GOALAREALENGTH 0.500000
#define PENALTYSPOT 2.000000
#define GOALWIDTH 2.000000
#define GOALDEPTH 0.600000
#define WHITEPOINTS_ALLOWEDERROR 0.18000
#define FIELDMAP_SHARPNESS 10.000000
/*---field dependent parameters end---*/

/* fieldline distance lookup table parameters */
#define FIELDMAP_RESOLUTION         100
#define FIELDMAP_BORDERWIDTH        1
#define FIELDMAP_X                  ((int)((FIELDWIDTH+(FIELDMAP_BORDERWIDTH*2))*FIELDMAP_RESOLUTION+1.5))
#define FIELDMAP_Y                  ((int)((FIELDLENGTH+(FIELDMAP_BORDERWIDTH*2))*FIELDMAP_RESOLUTION+1.5))

/* fieldline margins for (sub)target clipping */
#define BORDERMARGIN_SMALL 0.3        /* standard margin of subtarget wrt fieldlines */
#define BORDERMARGIN_LARGE 0.5       /* margin of subtarget wrt fieldlines for active refbox tasks */
#define BORDERMARGIN_LARGE_TIME 5     /* [s] hysteresis time to keep margin_large */

/* object parameters */
#define BALLRADIUS                  0.11
#define ROBOTRADIUS                 0.25
#define BALLROBOTDIST               0.35

/* dither fraction on ipcomm_shared_dusleep */
#define DITHER                      0.25

/* green intensity reference for adaptive shutter */
#define GREENREF 80

/* ball velocity estimator */
#define CLIP_LOWER_BALL_VELOCITY          0.25
#define CLIP_UPPER_BALL_VELOCITY          12.0

/* position & angle precision */
#define PRECISION_POSITION              0.14
#define PRECISION_ROTATION              0.045

/* port data */
#define PORT_STRATEGY_TUNING        9017
#define PORT_MOTION_TUNING          9018
#define PORT_WORLDMODEL_TUNING      9019
#define PORT_VISION_TUNING          9020

#define MAGENTA                     0
#define CYAN                        1

/* model samplerates */
#define MOTION_SAMPLE_RATE          1000
#define VISION_SAMPLE_RATE          32
#define WORLDMODEL_SAMPLE_RATE      10

#define MOTION_SAMPLE_TIME          1.0/MOTION_SAMPLE_RATE
#define VISION_SAMPLE_TIME          1.0/VISION_SAMPLE_RATE
#define WORLDMODEL_SAMPLE_TIME      1.0/WORLDMODEL_SAMPLE_RATE

#define RATE_MOTION                 1000
#define RATE_VISION                 32
#define RATE_STRATEGY               1000
#define RATE_WORLDMODEL             1000

/* maximum delay time buffering in matching pose vision and encoders in [s] */
#define MAX_DELAY_BUFFER            0.3
/* multicast communication
 * - maximum age time in milliseconds (ms) of the last message of the agent; if the 
 *   last communication is older than this age, the agent is considered no longer alive.
 * - number of obstacles accounted for in the communication (and the greenfield)
 * - number of objects accounted for in the communication (and the greenfield)
 */
#define MCCOMM_TIMEOUT_TIME         2000
#define MCCOMM_TIMEOUT_TIME_REFBOX  5000    /* larger than MCCOMM_TIMEOUT_TIME, as refbox_gui and _client only update at a rate of 1 [s] */
#define MCCOMM_TIMEOUT_TIME_APPMAN  5000

/*capture times
 * - determinesubtarget.c
 */
#undef CT_ENABLE /* Change to #undef to remove function */
#define CT_BUFFERLENGTH 10000
#define CT_TIMESPERSAMPLE 2

/* strategy
 * - dynamic role switching hysteresis time [s]
 */
#define S_TIMEOUT_CLIENT            0

/* battery voltage monitoring */
#define MAX_BAT_VOLT                26.0
#define MIN_BAT_VOLT                20.0

/* path planning */
#define SAFETYDISTANCE_LOCAL_OBJ        0.25    /*[m] safety range of the robot wrt other (local) objects*/
#define DRIBBLE_ANGLE_OFFSET            0.2     /*[rad] additional offset in subtarget angle wrt objects*/
#define SCRUM_SIDE_MOVEMENT             0.5     /*[m] subtarget movement during scrum if driving backward is no option*/
#define SUBTARGET_UPDATE_DELAY_DRIBBLE  33      /*[ms] subtarget update delay during dribble*/

/* general constants */
#define PI                              3.14159265358979    /*pi value*/

#endif
