#define S_FUNCTION_NAME localisationaid
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"
#include "math.h"
#include "../../Global_par/constants.h"

#define DISTANCE_TO_T_SPOT 2
#define DISTANCE 1

static void mdlInitializeSizes(SimStruct *S)
{
    /*PARAMETERS*/
    ssSetNumSFcnParams(S, 0);
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }
    
    /*INPUT PORTS*/
    if (!ssSetNumInputPorts(S, 4)) return;
    
    ssSetInputPortWidth(S, 0, 3);
    ssSetInputPortDataType(S, 0, SS_DOUBLE);
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortRequiredContiguous(S, 0, 1);
    
    ssSetInputPortWidth(S, 1, 1);
    ssSetInputPortDataType(S, 1, SS_INT32);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
    ssSetInputPortRequiredContiguous(S, 1, 1);
    
    ssSetInputPortWidth(S, 2, 1);
    ssSetInputPortDataType(S, 2, SS_INT32);
    ssSetInputPortDirectFeedThrough(S, 2, 1);
    ssSetInputPortRequiredContiguous(S, 2, 1);
    
    ssSetInputPortWidth(S, 3, 1);
    ssSetInputPortDataType(S, 3, SS_INT32);
    ssSetInputPortDirectFeedThrough(S, 3, 1);
    ssSetInputPortRequiredContiguous(S, 3, 1);
    
    
    
    /*OUTPUT PORTS*/
    if (!ssSetNumOutputPorts(S, 2)) return;
    
    ssSetOutputPortWidth(S, 0, 3);
    ssSetOutputPortDataType(S, 0, SS_DOUBLE);
    
    ssSetOutputPortWidth(S, 1, 1);
    ssSetOutputPortDataType(S, 1, SS_INT32);
}

static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}

static void mdlOutputs(SimStruct *S, int_T tid)
{
    /*inputs*/
    const real_T *posevision       = (const real_T*) ssGetInputPortSignal(S,0);
    const int_T *posefound         = (const int_T*) ssGetInputPortSignal(S,1);
    const int_T *task_ID           = (const int_T*) ssGetInputPortSignal(S,2);
    const int_T *robot_active     = (const int_T*) ssGetInputPortSignal(S,3);
    
    
    /*outputs*/
    real_T *posevision_output = (real_T*) ssGetOutputPortSignal(S,0);
    int_T *posefound_output  = (int_T*) ssGetOutputPortSignal(S,1);
    
    
    /*keeper localisation is only trusted on the home side ( till penaltyspot )    */
    
    
    
    /* homegoal is blue */
    
    
    if ( task_ID[0]==ROLE_GOALKEEPER && posefound[0]==1 && posevision[1] > (FIELDLENGTH*0.5 - PENALTYAREALENGTH-DISTANCE) )
    {
        posevision_output[0]=0;
        posevision_output[1]=0;
        posevision_output[2]=0;
        posefound_output[0]=0;
    }
    else
    {
        posevision_output[0]=posevision[0];
        posevision_output[1]=posevision[1];
        posevision_output[2]=posevision[2];
        posefound_output[0]=posefound[0];
    }
    
    
    
    
    
    /* if the root is not active, the self localisation is set to the T-spot*/
    if (robot_active[0] == 0)
    {
        if (posefound[0]==1 && fabs(posevision[1])<DISTANCE_TO_T_SPOT && fabs(posevision[0])>FIELDWIDTH*.5-DISTANCE_TO_T_SPOT)
        {
            posevision_output[0]=posevision[0];
            posevision_output[1]=posevision[1];
            posevision_output[2]=posevision[2];
            posefound_output[0]=posefound[0];
        }
        else
        {
            posevision_output[0] = FIELDWIDTH*.5;
            posevision_output[1] = 0;
            posevision_output[2] = PI*.5;
            posefound_output[0] = 1;
        }
    }
    
    
    
}

#undef MDL_INITIALIZE_CONDITIONS   /* Change to #undef to remove function */
#undef MDL_UPDATE  /* Change to #undef to remove function */
#undef MDL_DERIVATIVES  /* Change to #undef to remove function */

static void mdlTerminate(SimStruct *S)
{
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
